# Copyright 2017 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Script for generating pre-rendered frames used in integration tests."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# Internal dependencies.
from absl import app
from dm_control.mujoco.testing import image_utils
import six


def main(argv):
  del argv  # Unused.
  for sequence in six.itervalues(image_utils.SEQUENCES):
    sequence.save()

if __name__ == '__main__':
  app.run(main)



# Copyright 2017 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Python bindings and wrapper classes for MuJoCo."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function


from dm_control.mujoco.wrapper import mjbindings

from dm_control.mujoco.wrapper.core import callback_context
from dm_control.mujoco.wrapper.core import enable_timer

from dm_control.mujoco.wrapper.core import Error

from dm_control.mujoco.wrapper.core import get_schema

from dm_control.mujoco.wrapper.core import MjData
from dm_control.mujoco.wrapper.core import MjModel
from dm_control.mujoco.wrapper.core import MjrContext
from dm_control.mujoco.wrapper.core import MjvCamera
from dm_control.mujoco.wrapper.core import MjvFigure
from dm_control.mujoco.wrapper.core import MjvOption
from dm_control.mujoco.wrapper.core import MjvPerturb
from dm_control.mujoco.wrapper.core import MjvScene

from dm_control.mujoco.wrapper.core import save_last_parsed_model_to_xml
from dm_control.mujoco.wrapper.core import set_callback

from dm_control.mujoco.wrapper.core import UnmanagedMjrContext



# Copyright 2019 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Multi-agent MuJoCo soccer environment."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from dm_control import composer
from dm_control.locomotion import walkers
from dm_control.locomotion.soccer.boxhead import BoxHead
from dm_control.locomotion.soccer.initializers import Initializer
from dm_control.locomotion.soccer.initializers import UniformInitializer
from dm_control.locomotion.soccer.observables import CoreObservablesAdder
from dm_control.locomotion.soccer.observables import InterceptionObservablesAdder
from dm_control.locomotion.soccer.observables import MultiObservablesAdder
from dm_control.locomotion.soccer.observables import ObservablesAdder
from dm_control.locomotion.soccer.pitch import Pitch
from dm_control.locomotion.soccer.pitch import RandomizedPitch
from dm_control.locomotion.soccer.soccer_ball import SoccerBall
from dm_control.locomotion.soccer.task import Task
from dm_control.locomotion.soccer.team import Player
from dm_control.locomotion.soccer.team import Team

import enum
from six.moves import range


_RGBA_BLUE = [.1, .1, .8, 1.]
_RGBA_RED = [.8, .1, .1, 1.]


class WalkerType(enum.Enum):
  BOXHEAD = 0
  ANT = 1


def _make_walker(name, walker_id, marker_rgba, walker_type=WalkerType.BOXHEAD):
  """Construct a BoxHead walker."""
  if walker_type == WalkerType.BOXHEAD:
    return BoxHead(
        name=name,
        walker_id=walker_id,
        marker_rgba=marker_rgba,
    )
  if walker_type == WalkerType.ANT:
    return walkers.Ant(name=name, marker_rgba=marker_rgba)
  raise ValueError("Unrecognized walker type: %s" % walker_type)


def _make_players(team_size, walker_type):
  """Construct home and away teams each of `team_size` players."""
  home_players = []
  away_players = []
  for i in range(team_size):
    home_walker = _make_walker("home%d" % i, i, _RGBA_BLUE, walker_type)
    home_players.append(Player(Team.HOME, home_walker))

    away_walker = _make_walker("away%d" % i, i, _RGBA_RED, walker_type)
    away_players.append(Player(Team.AWAY, away_walker))
  return home_players + away_players


def load(team_size,
         time_limit=45.,
         random_state=None,
         disable_walker_contacts=False,
         walker_type=WalkerType.BOXHEAD):
  """Construct `team_size`-vs-`team_size` soccer environment.

  Args:
    team_size: Integer, the number of players per team. Must be between 1 and
      11.
    time_limit: Float, the maximum duration of each episode in seconds.
    random_state: (optional) an int seed or `np.random.RandomState` instance.
    disable_walker_contacts: (optional) if `True`, disable physical contacts
      between walkers.
    walker_type: the type of walker to instantiate in the environment.

  Returns:
    A `composer.Environment` instance.

  Raises:
    ValueError: If `team_size` is not between 1 and 11.
    ValueError: If `walker_type` is not recognized.
  """

  return composer.Environment(
      task=Task(
          players=_make_players(team_size, walker_type),
          arena=RandomizedPitch(
              min_size=(32, 24), max_size=(48, 36), keep_aspect_ratio=True),
          disable_walker_contacts=disable_walker_contacts),
      time_limit=time_limit,
      random_state=random_state)



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""A global registry of constructors for manipulation environments."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from dm_control.utils import containers

_ALL_CONSTRUCTORS = containers.TaggedTasks(allow_overriding_keys=False)

add = _ALL_CONSTRUCTORS.add
get_constructor = _ALL_CONSTRUCTORS.__getitem__
get_all_names = _ALL_CONSTRUCTORS.keys
get_tags = _ALL_CONSTRUCTORS.tags
get_names_by_tag = _ALL_CONSTRUCTORS.tagged

# This disables the check that prevents the same task constructor name from
# being added to the container more than once. This is done in order to allow
# individual task modules to be reloaded without also reloading `registry.py`
# first (e.g. when "hot-reloading" environments using IPython's `autoreload`
# extension).


def done_importing_tasks():
  _ALL_CONSTRUCTORS.allow_overriding_keys = True



# Copyright 2017 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Misc helper functions needed by autowrap.py."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import collections
import keyword
import re

import six
from six.moves import builtins

_MJXMACRO_SUFFIX = "_POINTERS"
_PYTHON_RESERVED_KEYWORDS = set(keyword.kwlist + dir(builtins))
if not six.PY2:
  _PYTHON_RESERVED_KEYWORDS.add("buffer")


class Indenter(object):
  r"""Callable context manager for tracking string indentation levels.

  Args:
    level: The initial indentation level.
    indent_str: The string used to indent each line.

  Example usage:

  ```python
  idt = Indenter()
  s = idt("level 0\n")
  with idt:
    s += idt("level 1\n")
    with idt:
      s += idt("level 2\n")
    s += idt("level 1 again\n")
  s += idt("back to level 0\n")
  print(s)
  ```
  """

  def __init__(self, level=0, indent_str="  "):
    self.indent_str = indent_str
    self.level = level

  def __enter__(self):
    self.level += 1
    return self

  def __exit__(self, type_, value, traceback):
    self.level -= 1

  def __call__(self, string):
    return indent(string, self.level, self.indent_str)


def indent(s, n=1, indent_str="  "):
  """Inserts `n * indent_str` at the start of each non-empty line in `s`."""
  p = n * indent_str
  return "".join((p + l) if l.lstrip() else l for l in s.splitlines(True))


class UniqueOrderedDict(collections.OrderedDict):
  """Subclass of `OrderedDict` that enforces the uniqueness of keys."""

  def __setitem__(self, k, v):
    if k in self:
      raise ValueError("Key '{}' already exists.".format(k))
    super(UniqueOrderedDict, self).__setitem__(k, v)


def macro_struct_name(name, suffix=None):
  """Converts mjxmacro struct names, e.g. "MJDATA_POINTERS" to "mjdata"."""
  if suffix is None:
    suffix = _MJXMACRO_SUFFIX
  return name[:-len(suffix)].lower()


def is_macro_pointer(name):
  """Returns True if the mjxmacro struct name contains pointer sizes."""
  return name.endswith(_MJXMACRO_SUFFIX)


def mangle_varname(s):
  """Append underscores to ensure that `s` is not a reserved Python keyword."""
  while s in _PYTHON_RESERVED_KEYWORDS:
    s += "_"
  return s


def mangle_struct_typename(s):
  """Strip leading underscores and make uppercase."""
  return s.lstrip("_").upper()


def mangle_comment(s):
  """Strip extraneous whitespace, add full-stops at end of each line."""
  if not isinstance(s, six.string_types):
    return "\n".join(mangle_comment(line) for line in s)
  elif not s:
    return "<no header comment found>."
  else:
    out = "\n".join(" ".join(line.split()) for line in s.splitlines())
    if not out.endswith("."):
      out += "."
    return out


def camel_case(s):
  """Convert a snake_case string (maybe with lowerCaseFirst) to CamelCase."""
  tokens = re.sub(r"([A-Z])", r" \1", s.replace("_", " ")).split()
  return "".join(w.title() for w in tokens)


def try_coerce_to_num(s, try_types=(int, float)):
  """Try to coerce string to Python numeric type, return None if empty."""
  if not s:
    return None
  for try_type in try_types:
    try:
      return try_type(s.rstrip("UuFf"))
    except (ValueError, AttributeError):
      continue
  return s


def recursive_dict_lookup(key, try_dict, max_depth=10):
  """Recursively map dictionary keys to values."""
  if max_depth < 0:
    raise KeyError("Maximum recursion depth exceeded")
  while key in try_dict:
    key = try_dict[key]
    return recursive_dict_lookup(key, try_dict, max_depth - 1)
  return key


def comment_line(string, width=79, fill_char="-"):
  """Wraps `string` in a padded comment line."""
  return "# {0:{2}^{1}}\n".format(string, width - 2, fill_char)



# Copyright 2017 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Reacher domain."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import collections

from dm_control import mujoco
from dm_control.rl import control
from dm_control.suite import base
from dm_control.suite import common
from dm_control.suite.utils import randomizers
from dm_control.utils import containers
from dm_control.utils import rewards
import numpy as np

SUITE = containers.TaggedTasks()
_DEFAULT_TIME_LIMIT = 20
_BIG_TARGET = .05
_SMALL_TARGET = .015


def get_model_and_assets():
  """Returns a tuple containing the model XML string and a dict of assets."""
  return common.read_model('reacher.xml'), common.ASSETS


@SUITE.add('benchmarking', 'easy')
def easy(time_limit=_DEFAULT_TIME_LIMIT, random=None, environment_kwargs=None):
  """Returns reacher with sparse reward with 5e-2 tol and randomized target."""
  physics = Physics.from_xml_string(*get_model_and_assets())
  task = Reacher(target_size=_BIG_TARGET, random=random)
  environment_kwargs = environment_kwargs or {}
  return control.Environment(
      physics, task, time_limit=time_limit, **environment_kwargs)


@SUITE.add('benchmarking')
def hard(time_limit=_DEFAULT_TIME_LIMIT, random=None, environment_kwargs=None):
  """Returns reacher with sparse reward with 1e-2 tol and randomized target."""
  physics = Physics.from_xml_string(*get_model_and_assets())
  task = Reacher(target_size=_SMALL_TARGET, random=random)
  environment_kwargs = environment_kwargs or {}
  return control.Environment(
      physics, task, time_limit=time_limit, **environment_kwargs)


class Physics(mujoco.Physics):
  """Physics simulation with additional features for the Reacher domain."""

  def finger_to_target(self):
    """Returns the vector from target to finger in global coordinates."""
    return (self.named.data.geom_xpos['target', :2] -
            self.named.data.geom_xpos['finger', :2])

  def finger_to_target_dist(self):
    """Returns the signed distance between the finger and target surface."""
    return np.linalg.norm(self.finger_to_target())


class Reacher(base.Task):
  """A reacher `Task` to reach the target."""

  def __init__(self, target_size, random=None):
    """Initialize an instance of `Reacher`.

    Args:
      target_size: A `float`, tolerance to determine whether finger reached the
          target.
      random: Optional, either a `numpy.random.RandomState` instance, an
        integer seed for creating a new `RandomState`, or None to select a seed
        automatically (default).
    """
    self._target_size = target_size
    super(Reacher, self).__init__(random=random)

  def initialize_episode(self, physics):
    """Sets the state of the environment at the start of each episode."""
    physics.named.model.geom_size['target', 0] = self._target_size
    randomizers.randomize_limited_and_rotational_joints(physics, self.random)

    # Randomize target position
    angle = self.random.uniform(0, 2 * np.pi)
    radius = self.random.uniform(.05, .20)
    physics.named.model.geom_pos['target', 'x'] = radius * np.sin(angle)
    physics.named.model.geom_pos['target', 'y'] = radius * np.cos(angle)

    super(Reacher, self).initialize_episode(physics)

  def get_observation(self, physics):
    """Returns an observation of the state and the target position."""
    obs = collections.OrderedDict()
    obs['position'] = physics.position()
    obs['to_target'] = physics.finger_to_target()
    obs['velocity'] = physics.velocity()
    return obs

  def get_reward(self, physics):
    radii = physics.named.model.geom_size[['target', 'finger'], 0].sum()
    return rewards.tolerance(physics.finger_to_target_dist(), (0, radii))



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Module defining the abstract entity class."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import abc
import collections
import os
import weakref

from absl import logging
from dm_control import mjcf
from dm_control.composer import define
from dm_control.mujoco.wrapper import mjbindings
import numpy as np
import six

_OPTION_KEYS = set(['update_interval', 'buffer_size', 'delay', 'aggregator',
                    'corruptor', 'enabled'])

_NO_ATTACHMENT_FRAME = 'No attachment frame found.'


# The component order differs from that used by the open-source `tf` package.
def _multiply_quaternions(quat1, quat2):
  result = np.empty_like(quat1)
  mjbindings.mjlib.mju_mulQuat(result, quat1, quat2)
  return result


def _rotate_vector(vec, quat):
  """Rotates a vector by the given quaternion."""
  result = np.empty_like(vec)
  mjbindings.mjlib.mju_rotVecQuat(result, vec, quat)
  return result


class _ObservableKeys(object):
  """Helper object that implements the `observables.dict_keys` functionality."""

  def __init__(self, entity, observables):
    self._entity = entity
    self._observables = observables

  def __getattr__(self, name):
    try:
      model_identifier = self._entity.mjcf_model.full_identifier
    except AttributeError:
      raise ValueError('cannot retrieve the full identifier of mjcf_model')
    return os.path.join(model_identifier, name)

  def __dir__(self):
    out = set(self._observables.keys())
    out.update(dir(super(_ObservableKeys, self)))
    return list(out)


class Observables(object):
  """Base-class for Entity observables.

  Subclasses should declare getter methods annotated with @define.observable
  decorator and returning an observable object.
  """

  def __init__(self, entity):
    self._entity = weakref.proxy(entity)

    self._observables = collections.OrderedDict()
    self._keys_helper = _ObservableKeys(self._entity, self._observables)

    # Ensure consistent ordering.
    for attr_name in sorted(dir(type(self))):
      type_attr = getattr(type(self), attr_name)
      if isinstance(type_attr, define.observable):
        self._observables[attr_name] = getattr(self, attr_name)

  @property
  def dict_keys(self):
    return self._keys_helper

  def as_dict(self, fully_qualified=True):
    """Returns an OrderedDict of observables belonging to this Entity.

    The returned observables will include any added using the _add_observable
    method, as well as any generated by a method decorated with the
    @define.observable annotation.

    Args:
      fully_qualified: (bool) Whether the dict keys should be prefixed with the
        parent entity's full model identifier.
    """

    if fully_qualified:
      # We need to make sure that this property doesn't raise an AttributeError,
      # otherwise __getattr__ is executed and we get a very funky error.
      try:
        model_identifier = self._entity.mjcf_model.full_identifier
      except AttributeError:
        raise ValueError('cannot retrieve the full identifier of mjcf_model')

      return collections.OrderedDict(
          [(os.path.join(model_identifier, name), observable)
           for name, observable in six.iteritems(self._observables)])
    else:
      # Return a copy to prevent dict being edited.
      return self._observables.copy()

  def get_observable(self, name, name_fully_qualified=False):
    """Returns the observable with the given name.

    Args:
      name: (str) The identifier of the observable.
      name_fully_qualified: (bool) Whether the provided name is prefixed by the
        model's full identifier.
    """

    if name_fully_qualified:
      try:
        model_identifier = self._entity.mjcf_model.full_identifier
      except AttributeError:
        raise ValueError('cannot retrieve the full identifier of mjcf_model')
      return self._observables[name.replace(model_identifier, '')]
    else:
      return self._observables[name]

  def set_options(self, options):
    """Configure Observables with an options dict.

    Args:
      options: A dict of dicts of configuration options keyed on
        observable names, or a dict of configuration options, which will
        propagate those options to all observables.
    """
    if options is None:
      options = {}
    elif options.keys() and set(options.keys()).issubset(_OPTION_KEYS):
      options = dict([(key, options) for key in self._observables.keys()])

    for obs_key, obs_options in six.iteritems(options):
      try:
        obs = self._observables[obs_key]
      except KeyError:
        raise KeyError('No observable with name {!r}'.format(obs_key))
      obs.configure(**obs_options)

  def enable_all(self):
    """Enable all observables of this entity."""
    for obs in self._observables.values():
      obs.enabled = True

  def disable_all(self):
    """Disable all observables of this entity."""
    for obs in self._observables.values():
      obs.enabled = False

  def add_observable(self, name, observable, enabled=True):
    self._observables[name] = observable
    self._observables[name].enabled = enabled


@six.add_metaclass(abc.ABCMeta)
class FreePropObservableMixin(object):
  """Enforce observables of a free-moving object."""

  @abc.abstractproperty
  def position(self):
    pass

  @abc.abstractproperty
  def orientation(self):
    pass

  @abc.abstractproperty
  def linear_velocity(self):
    pass

  @abc.abstractproperty
  def angular_velocity(self):
    pass


@six.add_metaclass(abc.ABCMeta)
class Entity(object):
  """The abstract base class for an entity in a Composer environment."""

  def __init__(self, *args, **kwargs):
    """Entity constructor.

    Subclasses should not override this method, instead implement a _build
    method.

    Args:
      *args: Arguments passed through to the _build method.
      **kwargs: Keyword arguments. Passed through to the _build method, apart
        from the following.
          `observable_options`: A dictionary of Observable
            configuration options.
    """
    self._post_init_hooks = []

    self._parent = None
    self._attached = []

    try:
      observable_options = kwargs.pop('observable_options')
    except KeyError:
      observable_options = None

    self._build(*args, **kwargs)
    self._observables = self._build_observables()
    self._observables.set_options(observable_options)

  @abc.abstractmethod
  def _build(self, *args, **kwargs):
    """Entity initialization method to be overridden by subclasses."""
    raise NotImplementedError

  def _build_observables(self):
    """Entity observables initialization method.

    Returns:
      An object subclassing the Observables class.
    """
    return Observables(self)

  def iter_entities(self, exclude_self=False):
    """An iterator that recursively iterates through all attached entities.

    Args:
      exclude_self: (optional) Whether to exclude this `Entity` itself from the
        iterator.

    Yields:
      If `exclude_self` is `False`, the first value yielded is this Entity
      itself. The following Entities are then yielded recursively in a
      depth-first fashion, following the order in which the Entities are
      attached.
    """
    if not exclude_self:
      yield self
    for attached_entity in self._attached:
      for attached_entity_of_attached_entity in attached_entity.iter_entities():
        yield attached_entity_of_attached_entity

  @property
  def observables(self):
    """The observables defined by this entity."""
    return self._observables

  def initialize_episode_mjcf(self, random_state):
    """Callback executed when the MJCF model is modified between episodes."""
    pass

  def after_compile(self, physics, random_state):
    """Callback executed after the Mujoco Physics is recompiled."""
    pass

  def initialize_episode(self, physics, random_state):
    """Callback executed during episode initialization."""
    pass

  def before_step(self, physics, random_state):
    """Callback executed before an agent control step."""
    pass

  def before_substep(self, physics, random_state):
    """Callback executed before a simulation step."""
    pass

  def after_substep(self, physics, random_state):
    """A callback which is executed after a simulation step."""
    pass

  def after_step(self, physics, random_state):
    """Callback executed after an agent control step."""
    pass

  @abc.abstractproperty
  def mjcf_model(self):
    raise NotImplementedError

  def attach(self, entity, attach_site=None):
    """Attaches an `Entity` without any additional degrees of freedom.

    Args:
      entity: The `Entity` to attach.
      attach_site: (optional) The site to which to attach the entity's model. If
          not set, defaults to self.attachment_site.

    Returns:
      The frame of the attached model.
    """

    if attach_site is None:
      attach_site = self.attachment_site

    frame = attach_site.attach(entity.mjcf_model)
    self._attached.append(entity)
    entity._parent = weakref.ref(self)  # pylint: disable=protected-access
    return frame

  def detach(self):
    """Detaches this entity if it has previously been attached."""
    if self._parent is not None:
      parent = self._parent()
      if parent:  # Weakref might dereference to None during garbage collection.
        self.mjcf_model.detach()
        parent._attached.remove(self)  # pylint: disable=protected-access
      self._parent = None
    else:
      raise RuntimeError('Cannot detach an entity that is not attached.')

  @property
  def parent(self):
    """Returns the `Entity` to which this entity is attached, or `None`."""
    return self._parent() if self._parent else None

  @property
  def attachment_site(self):
    return self.mjcf_model

  @property
  def root_body(self):
    if self.parent:
      return mjcf.get_attachment_frame(self.mjcf_model)
    else:
      return self.mjcf_model.worldbody

  def global_vector_to_local_frame(self, physics, vec_in_world_frame):
    """Linearly transforms a world-frame vector into entity's local frame.

    Note that this function does not perform an affine transformation of the
    vector. In other words, the input vector is assumed to be specified with
    respect to the same origin as this entity's local frame. This function
    can also be applied to matrices whose innermost dimensions are either 2 or
    3. In this case, a matrix with the same leading dimensions is returned
    where the innermost vectors are replaced by their values computed in the
    local frame.

    Args:
      physics: An `mjcf.Physics` instance.
      vec_in_world_frame: A NumPy array with last dimension of shape (2,) or
      (3,) that represents a vector quantity in the world frame.

    Returns:
      The same quantity as `vec_in_world_frame` but reexpressed in this
      entity's local frame. The returned np.array has the same shape as
      np.asarray(vec_in_world_frame).

    Raises:
      ValueError: if `vec_in_world_frame` does not have shape ending with (2,)
        or (3,).
    """
    vec_in_world_frame = np.asarray(vec_in_world_frame)

    xmat = np.reshape(physics.bind(self.root_body).xmat, (3, 3))
    # The ordering of the np.dot is such that the transformation holds for any
    # matrix whose final dimensions are (2,) or (3,).
    if vec_in_world_frame.shape[-1] == 2:
      return np.dot(vec_in_world_frame, xmat[:2, :2])
    elif vec_in_world_frame.shape[-1] == 3:
      return np.dot(vec_in_world_frame, xmat)
    else:
      raise ValueError('`vec_in_world_frame` should have shape with final '
                       'dimension 2 or 3: got {}'.format(
                           vec_in_world_frame.shape))

  def global_xmat_to_local_frame(self, physics, xmat):
    """Transforms another entity's `xmat` into this entity's local frame.

    This function takes another entity's (E) xmat, which is an SO(3) matrix
    from E's frame to the world frame, and turns it to a matrix that transforms
    from E's frame into this entity's local frame.

    Args:
      physics: An `mjcf.Physics` instance.
      xmat: A NumPy array of shape (3, 3) or (9,) that represents another
        entity's xmat.

    Returns:
      The `xmat` reexpressed in this entity's local frame. The returned
      np.array has the same shape as np.asarray(xmat).

    Raises:
      ValueError: if `xmat` does not have shape (3, 3) or (9,).
    """
    xmat = np.asarray(xmat)

    input_shape = xmat.shape
    if xmat.shape == (9,):
      xmat = np.reshape(xmat, (3, 3))

    self_xmat = np.reshape(physics.bind(self.root_body).xmat, (3, 3))
    if xmat.shape == (3, 3):
      return np.reshape(np.dot(self_xmat.T, xmat), input_shape)
    else:
      raise ValueError('`xmat` should have shape (3, 3) or (9,): got {}'.format(
          xmat.shape))

  def get_pose(self, physics):
    """Get the position and orientation of this entity relative to its parent.

    Note that the semantics differ slightly depending on whether or not the
    entity has a free joint:

    * If it has a free joint the position and orientation are always given in
      global coordinates.
    * If the entity is fixed or attached with a different joint type then the
      position and orientation are given relative to the parent frame.

    For entities that are either attached directly to the worldbody, or to other
    entities that are positioned at the global origin (e.g. the arena) the
    global and relative poses are equivalent.

    Args:
      physics: An instance of `mjcf.Physics`.

    Returns:
      A 2-tuple where the first entry is a (3,) numpy array representing the
      position and the second is a (4,) numpy array representing orientation as
      a quaternion.

    Raises:
      RuntimeError: If the entity is not attached.
    """
    root_joint = mjcf.get_frame_freejoint(self.mjcf_model)
    if root_joint:
      position = physics.bind(root_joint).qpos[:3]
      quaternion = physics.bind(root_joint).qpos[3:]
    else:
      attachment_frame = mjcf.get_attachment_frame(self.mjcf_model)
      if attachment_frame is None:
        raise RuntimeError(_NO_ATTACHMENT_FRAME)
      position = physics.bind(attachment_frame).pos
      quaternion = physics.bind(attachment_frame).quat
    return position, quaternion

  def set_pose(self, physics, position=None, quaternion=None):
    """Sets position and/or orientation of this entity relative to its parent.

    If the entity is attached with a free joint, this method will set the
    respective DoFs of the joint. If the entity is either fixed or attached with
    a different joint type, this method will update the position and/or
    quaternion of the attachment frame.

    Note that the semantics differ slightly between the two cases: the DoFs of a
    free body are specified in global coordinates, whereas the position of a
    non-free body is specified in relative coordinates with respect to the
    parent frame. However, for entities that are either attached directly to the
    worldbody, or to other entities that are positioned at the global origin
    (e.g. the arena), there is no difference between the two cases.

    Args:
      physics: An instance of `mjcf.Physics`.
      position: (optional) A NumPy array of size 3.
      quaternion: (optional) A NumPy array of size 4.

    Raises:
      RuntimeError: If the entity is not attached.
    """
    root_joint = mjcf.get_frame_freejoint(self.mjcf_model)
    if root_joint:
      if position is not None:
        physics.bind(root_joint).qpos[:3] = position
      if quaternion is not None:
        physics.bind(root_joint).qpos[3:] = quaternion
    else:
      attachment_frame = mjcf.get_attachment_frame(self.mjcf_model)
      if attachment_frame is None:
        raise RuntimeError(_NO_ATTACHMENT_FRAME)
      if position is not None:
        physics.bind(attachment_frame).pos = position
      if quaternion is not None:
        normalised_quaternion = quaternion / np.linalg.norm(quaternion)
        physics.bind(attachment_frame).quat = normalised_quaternion

  def shift_pose(self,
                 physics,
                 position=None,
                 quaternion=None,
                 rotate_velocity=False):
    """Shifts the position and/or orientation from its current configuration.

    This is a convenience function that performs the same operation as
    `set_pose`, but where the specified `position` is added to the current
    position, and the specified `quaternion` is premultiplied to the current
    quaternion.

    Args:
      physics: An instance of `mjcf.Physics`.
      position: (optional) A NumPy array of size 3.
      quaternion: (optional) A NumPy array of size 4.
      rotate_velocity: (optional) A bool, whether to shift the current linear
        velocity along with the pose. This will rotate the current linear
        velocity, which is expressed relative to the world frame. The angular
        velocity, which is expressed relative to the local frame is left
        unchanged.

    Raises:
      RuntimeError: If the entity is not attached.
    """
    current_position, current_quaternion = self.get_pose(physics)
    new_position, new_quaternion = None, None
    if position is not None:
      new_position = current_position + position
    if quaternion is not None:
      quaternion = np.array(quaternion, dtype=np.float64, copy=False)
      new_quaternion = _multiply_quaternions(quaternion, current_quaternion)
      root_joint = mjcf.get_frame_freejoint(self.mjcf_model)
      if root_joint and rotate_velocity:
        # Rotate the linear velocity. The angular velocity (qvel[3:])
        # is left unchanged, as it is expressed in the local frame.
        # When rotatating the body frame the angular velocity already
        # tracks the rotation but the linear velocity does not.
        velocity = physics.bind(root_joint).qvel[:3]
        rotated_velocity = _rotate_vector(velocity, quaternion)
        self.set_velocity(physics, rotated_velocity)
    self.set_pose(physics, new_position, new_quaternion)

  def set_velocity(self, physics, velocity=None, angular_velocity=None):
    """Sets the linear velocity and/or angular velocity of this free entity.

    If the entity is attached with a free joint, this method will set the
    respective DoFs of the joint. Otherwise a warning is logged.

    Args:
      physics: An instance of `mjcf.Physics`.
      velocity: (optional) A NumPy array of size 3 specifying the
        linear velocity.
      angular_velocity: (optional) A NumPy array of size 3 specifying the
        angular velocity
    """
    root_joint = mjcf.get_frame_freejoint(self.mjcf_model)
    if root_joint:
      if velocity is not None:
        physics.bind(root_joint).qvel[:3] = velocity
      if angular_velocity is not None:
        physics.bind(root_joint).qvel[3:] = angular_velocity
    else:
      logging.warning('Cannot set velocity on Entity with no free joint.')

  def configure_joints(self, physics, position):
    """Configures this entity's internal joints.

    The default implementation of this method simply sets the `qpos` of all
    joints in this entity to the values specified in the `position` argument.
    Entity subclasses with actuated joints may override this method to achieve a
    stable reconfiguration of joint positions, for example the control signal
    of position actuators may be changed to match the new joint positions.

    Args:
      physics: An instance of `mjcf.Physics`.
      position: The desired position of this entity's joints.
    """
    joints = self.mjcf_model.find_all('joint', exclude_attachments=True)
    physics.bind(joints).qpos = position


class ModelWrapperEntity(Entity):
  """An entity class that wraps an MJCF model without any additional logic."""

  def _build(self, mjcf_model):
    self._mjcf_model = mjcf_model

  @property
  def mjcf_model(self):
    return self._mjcf_model



# Copyright 2017-2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Install script for setuptools."""

from distutils import cmd
from distutils import log
import fnmatch
import os
import platform
import subprocess
import sys

from setuptools import find_packages
from setuptools import setup
from setuptools.command import install
from setuptools.command import test

PLATFORM_SUFFIXES = {
    'Linux': 'linux',
    'Windows': 'win64',
    'Darwin': 'macos',
}
DEFAULT_HEADERS_DIR = '~/.mujoco/mujoco200_{}/include'.format(
    PLATFORM_SUFFIXES[platform.system()])

# Relative paths to the binding generator script and the output directory.
AUTOWRAP_PATH = 'dm_control/autowrap/autowrap.py'
MJBINDINGS_DIR = 'dm_control/mujoco/wrapper/mjbindings'

# We specify the header filenames explicitly rather than listing the contents
# of the `HEADERS_DIR` at runtime, since it will probably contain other stuff
# (e.g. `glfw.h`).
HEADER_FILENAMES = [
    'mjdata.h',
    'mjmodel.h',
    'mjrender.h',
    'mjui.h',
    'mjvisualize.h',
    'mjxmacro.h',
    'mujoco.h',
]


def _initialize_mjbindings_options(cmd_instance):
  """Set default values for options relating to `build_mjbindings`."""
  # A default value must be assigned to each user option here.
  cmd_instance.inplace = 0
  cmd_instance.headers_dir = DEFAULT_HEADERS_DIR


def _finalize_mjbindings_options(cmd_instance):
  """Post-process options relating to `build_mjbindings`."""
  headers_dir = os.path.expanduser(cmd_instance.headers_dir)
  header_paths = []
  for filename in HEADER_FILENAMES:
    full_path = os.path.join(headers_dir, filename)
    if not os.path.exists(full_path):
      raise IOError('Header file {!r} does not exist.'.format(full_path))
    header_paths.append(full_path)
  cmd_instance.header_paths = ' '.join(header_paths)


class BuildMJBindingsCommand(cmd.Command):
  """Runs `autowrap.py` to generate the low-level ctypes bindings for MuJoCo."""
  description = __doc__
  user_options = [
      # The format is (long option, short option, description).
      ('headers-dir=', None,
       'Path to directory containing MuJoCo headers.'),
      ('inplace=', None,
       'Place generated files in source directory rather than `build-lib`.'),
  ]
  boolean_options = ['inplace']

  def initialize_options(self):
    _initialize_mjbindings_options(self)

  def finalize_options(self):
    _finalize_mjbindings_options(self)

  def run(self):
    cwd = os.path.realpath(os.curdir)
    if self.inplace:
      dist_root = cwd
    else:
      build_cmd = self.get_finalized_command('build')
      dist_root = os.path.realpath(build_cmd.build_lib)
    output_dir = os.path.join(dist_root, MJBINDINGS_DIR)
    command = [
        sys.executable or 'python',
        AUTOWRAP_PATH,
        '--header_paths={}'.format(self.header_paths),
        '--output_dir={}'.format(output_dir)
    ]
    self.announce('Running command: {}'.format(command), level=log.DEBUG)
    try:
      # Prepend the current directory to $PYTHONPATH so that internal imports
      # in `autowrap` can succeed before we've installed anything.
      old_environ = os.environ.copy()
      new_pythonpath = [cwd]
      if 'PYTHONPATH' in old_environ:
        new_pythonpath.append(old_environ['PYTHONPATH'])
      os.environ['PYTHONPATH'] = ':'.join(new_pythonpath)
      subprocess.check_call(command)
    finally:
      os.environ = old_environ


class InstallCommand(install.install):
  """Runs 'build_mjbindings' before installation."""

  user_options = (
      install.install.user_options + BuildMJBindingsCommand.user_options)
  boolean_options = (
      install.install.boolean_options + BuildMJBindingsCommand.boolean_options)

  def initialize_options(self):
    install.install.initialize_options(self)
    _initialize_mjbindings_options(self)

  def finalize_options(self):
    install.install.finalize_options(self)
    _finalize_mjbindings_options(self)

  def run(self):
    self.reinitialize_command('build_mjbindings',
                              inplace=self.inplace,
                              headers_dir=self.headers_dir)
    self.run_command('build_mjbindings')
    install.install.run(self)


class TestCommand(test.test):
  """Prepends path to generated sources before running unit tests."""

  def run(self):
    # Generate ctypes bindings in-place so that they can be imported in tests.
    self.reinitialize_command('build_mjbindings', inplace=1)
    self.run_command('build_mjbindings')
    test.test.run(self)


def find_data_files(package_dir, patterns, excludes=()):
  """Recursively finds files whose names match the given shell patterns."""
  paths = set()

  def is_excluded(s):
    for exclude in excludes:
      if fnmatch.fnmatch(s, exclude):
        return True
    return False

  for directory, _, filenames in os.walk(package_dir):
    if is_excluded(directory):
      continue
    for pattern in patterns:
      for filename in fnmatch.filter(filenames, pattern):
        # NB: paths must be relative to the package directory.
        relative_dirpath = os.path.relpath(directory, package_dir)
        full_path = os.path.join(relative_dirpath, filename)
        if not is_excluded(full_path):
          paths.add(full_path)
  return list(paths)

setup(
    name='dm_control',
    version='0.0.318037100',
    description='Continuous control environments and MuJoCo Python bindings.',
    author='DeepMind',
    license='Apache License, Version 2.0',
    keywords='machine learning control physics MuJoCo AI',
    install_requires=[
        'absl-py>=0.7.0',
        'enum34; python_version < "3.4"',
        'dm-env',
        'dm-tree != 0.1.2',
        'future',
        'futures; python_version == "2.7"',
        'glfw',
        'lxml',
        'numpy',
        'pyopengl >= 3.1.4',
        'pyparsing',
        'setuptools',
        'scipy',
        'six',
    ],
    extras_require={
        'locomotion_mazes': ['labmaze'],
    },
    tests_require=[
        'mock',
        'nose',
        'pillow',
    ],
    test_suite='nose.collector',
    packages=find_packages(),
    package_data={
        'dm_control':
            find_data_files(
                package_dir='dm_control',
                patterns=['*.amc', '*.msh', '*.png', '*.skn', '*.stl', '*.xml'],
                excludes=[
                    '*/dog_assets/extras/*',
                    '*/kinova/meshes/*',  # Exclude non-decimated meshes.
                ]),
    },
    cmdclass={
        'build_mjbindings': BuildMJBindingsCommand,
        'install': InstallCommand,
        'test': TestCommand,
    },
    entry_points={},
)



# Copyright 2017 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Import core names of MuJoCo ctypes bindings."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from absl import logging

from dm_control.mujoco.wrapper.mjbindings import constants
from dm_control.mujoco.wrapper.mjbindings import enums
from dm_control.mujoco.wrapper.mjbindings import sizes
from dm_control.mujoco.wrapper.mjbindings import types
from dm_control.mujoco.wrapper.mjbindings import wrappers

# pylint: disable=g-import-not-at-top
try:
  from dm_control.mujoco.wrapper.mjbindings import functions
  from dm_control.mujoco.wrapper.mjbindings.functions import mjlib
  logging.info('MuJoCo library version is: %d', mjlib.mj_version())
except (IOError, OSError):
  logging.warning('mjbindings failed to import mjlib and other functions. '
                  'libmujoco.so may not be accessible.')



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""PyMJCF: an MJCF object-model library."""

from dm_control.mjcf.attribute import Asset

from dm_control.mjcf.base import Element

from dm_control.mjcf.constants import PREFIX_SEPARATOR

from dm_control.mjcf.element import RootElement

from dm_control.mjcf.export_with_assets import export_with_assets
from dm_control.mjcf.export_with_assets_as_zip import export_with_assets_as_zip

from dm_control.mjcf.parser import from_file
from dm_control.mjcf.parser import from_path
from dm_control.mjcf.parser import from_xml_string

from dm_control.mjcf.physics import Physics

from dm_control.mjcf.traversal_utils import get_attachment_frame
from dm_control.mjcf.traversal_utils import get_frame_freejoint
from dm_control.mjcf.traversal_utils import get_frame_joints
from dm_control.mjcf.traversal_utils import get_freejoint



# Copyright 2019 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Tests for index."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import math
from absl.testing import absltest
from absl.testing import parameterized
from dm_control.mujoco import math as mjmath
import numpy as np


class MathTest(parameterized.TestCase):

  def testQuatProd(self):
    np.testing.assert_allclose(
        mjmath.mj_quatprod([0., 1., 0., 0.], [0., 0., 1., 0.]),
        [0., 0., 0., 1.])
    np.testing.assert_allclose(
        mjmath.mj_quatprod([0., 0., 1., 0.], [0., 0., 0., 1.]),
        [0., 1., 0., 0.])
    np.testing.assert_allclose(
        mjmath.mj_quatprod([0., 0., 0., 1.], [0., 1., 0., 0.]),
        [0., 0., 1., 0.])

  def testQuat2Vel(self):
    np.testing.assert_allclose(
        mjmath.mj_quat2vel([0., 1., 0., 0.], 0.1), [math.pi / 0.1, 0., 0.])

  def testQuatNeg(self):
    np.testing.assert_allclose(
        mjmath.mj_quatneg([math.sqrt(0.5), math.sqrt(0.5), 0., 0.]),
        [math.sqrt(0.5), -math.sqrt(0.5), 0., 0.])

  def testQuatDiff(self):
    np.testing.assert_allclose(
        mjmath.mj_quatdiff([0., 1., 0., 0.], [0., 0., 1., 0.]),
        [0., 0., 0., -1.])

  def testEuler2Quat(self):
    np.testing.assert_allclose(
        mjmath.euler2quat(0., 0., 0.), [1., 0., 0., 0.])


if __name__ == '__main__':
  absltest.main()



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Tests for noises."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# Internal dependencies.
from absl.testing import absltest
from absl.testing import parameterized
from dm_control.composer.variation import deterministic
from dm_control.composer.variation import noises
from six.moves import range

NUM_ITERATIONS = 100


class NoisesTest(parameterized.TestCase):

  @parameterized.parameters(False, True)
  def testAdditive(self, use_constant_variation_object):
    amount = 2
    if use_constant_variation_object:
      variation = noises.Additive(deterministic.Constant(amount))
    else:
      variation = noises.Additive(amount)
    initial_value = 0
    current_value = initial_value
    for _ in range(NUM_ITERATIONS):
      current_value = variation(
          initial_value=initial_value, current_value=current_value)
      self.assertEqual(current_value, initial_value + amount)

  @parameterized.parameters(False, True)
  def testAdditiveCumulative(self, use_constant_variation_object):
    amount = 3
    if use_constant_variation_object:
      variation = noises.Additive(
          deterministic.Constant(amount), cumulative=True)
    else:
      variation = noises.Additive(amount, cumulative=True)
    initial_value = 1
    current_value = initial_value
    for i in range(NUM_ITERATIONS):
      current_value = variation(
          initial_value=initial_value, current_value=current_value)
      self.assertEqual(current_value, initial_value + amount * (i + 1))

  @parameterized.parameters(False, True)
  def testMultiplicative(self, use_constant_variation_object):
    amount = 23
    if use_constant_variation_object:
      variation = noises.Multiplicative(deterministic.Constant(amount))
    else:
      variation = noises.Multiplicative(amount)
    initial_value = 3
    current_value = initial_value
    for _ in range(NUM_ITERATIONS):
      current_value = variation(
          initial_value=initial_value, current_value=current_value)
      self.assertEqual(current_value, initial_value * amount)

  @parameterized.parameters(False, True)
  def testMultiplicativeCumulative(self, use_constant_variation_object):
    amount = 2
    if use_constant_variation_object:
      variation = noises.Multiplicative(
          deterministic.Constant(amount), cumulative=True)
    else:
      variation = noises.Multiplicative(amount, cumulative=True)
    initial_value = 3
    current_value = initial_value
    for i in range(NUM_ITERATIONS):
      current_value = variation(
          initial_value=initial_value, current_value=current_value)
      self.assertEqual(current_value, initial_value * amount ** (i + 1))

if __name__ == '__main__':
  absltest.main()



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Abstract base class for a Composer task."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import abc
import collections
import copy
import sys

from dm_control import mujoco
from dm_env import specs
import six
from six.moves import range


def _check_timesteps_divisible(control_timestep, physics_timestep):
  num_steps = control_timestep / physics_timestep
  rounded_num_steps = int(round(num_steps))
  if abs(num_steps - rounded_num_steps) > 1e-6:
    raise ValueError(
        'Control timestep should be an integer multiple of physics timestep'
        ': got {!r} and {!r}'.format(control_timestep, physics_timestep))
  return rounded_num_steps


@six.add_metaclass(abc.ABCMeta)
class Task(object):
  """Abstract base class for a Composer task."""

  @abc.abstractproperty
  def root_entity(self):
    """A `base.Entity` instance for this task."""
    raise NotImplementedError

  def iter_entities(self):
    return self.root_entity.iter_entities()

  @property
  def observables(self):
    """An OrderedDict of `control.Observable` instances for this task.

    Task subclasses should generally NOT override this property.

    This property is automatically computed by combining the observables dict
    provided by each `Entity` present in this task, and any additional
    observables returned via the `task_observables` property.

    To provide an observable to an agent, the task code should either set
    `enabled` property of an `Entity`-bound observable to `True`, or override
    the `task_observables` property to provide additional observables not bound
    to an `Entity`.

    Returns:
      An `collections.OrderedDict` mapping strings to instances of
      `control.Observable`.
    """
    # Make a shallow copy of the OrderedDict, not the Observables themselves.
    observables = copy.copy(self.task_observables)
    for entity in self.root_entity.iter_entities():
      observables.update(entity.observables.as_dict())
    return observables

  @property
  def task_observables(self):
    """An OrderedDict of task-specific `control.Observable` instances.

    A task should override this property if it wants to provide additional
    observables to the agent that are not already provided by any `Entity` that
    forms part of the task's model. For example, this may be used to provide
    observations that is derived from relative poses between two entities.

    Returns:
      An `collections.OrderedDict` mapping strings to instances of
      `control.Observable`.
    """
    return collections.OrderedDict()

  def after_compile(self, physics, random_state):
    """A callback which is executed after the Mujoco Physics is recompiled.

    Args:
      physics: An instance of `control.Physics`.
      random_state: An instance of `np.random.RandomState`.
    """
    pass

  def _check_root_entity(self, callee_name):
    try:
      _ = self.root_entity
    except:  # pylint: disable=bare-except
      err_type, err, tb = sys.exc_info()
      message = (
          'call to `{}` made before `root_entity` is available;\n'
          'original error message: {}'.format(callee_name, str(err)))
      six.reraise(err_type, err_type(message), tb)

  @property
  def control_timestep(self):
    """Returns the agent's control timestep for this task (in seconds)."""
    self._check_root_entity('control_timestep')
    if hasattr(self, '_control_timestep'):
      return self._control_timestep
    else:
      return self.physics_timestep

  @control_timestep.setter
  def control_timestep(self, new_value):
    """Changes the agent's control timestep for this task.

    Args:
      new_value: the new control timestep (in seconds).

    Raises:
      ValueError: if `new_value` is set and is not divisible by
        `physics_timestep`.
    """
    self._check_root_entity('control_timestep')
    _check_timesteps_divisible(new_value, self.physics_timestep)
    self._control_timestep = new_value

  @property
  def physics_timestep(self):
    """Returns the physics timestep for this task (in seconds)."""
    self._check_root_entity('physics_timestep')
    if self.root_entity.mjcf_model.option.timestep is None:
      return 0.002  # MuJoCo's default.
    else:
      return self.root_entity.mjcf_model.option.timestep

  @physics_timestep.setter
  def physics_timestep(self, new_value):
    """Changes the physics simulation timestep for this task.

    Args:
      new_value: the new simulation timestep (in seconds).

    Raises:
      ValueError: if `control_timestep` is set and is not divisible by
        `new_value`.
    """
    self._check_root_entity('physics_timestep')
    if hasattr(self, '_control_timestep'):
      _check_timesteps_divisible(self._control_timestep, new_value)
    self.root_entity.mjcf_model.option.timestep = new_value

  def set_timesteps(self, control_timestep, physics_timestep):
    """Changes the agent's control timestep and physics simulation timestep.

    This is equivalent to modifying `control_timestep` and `physics_timestep`
    simultaneously. The divisibility check is performed between the two
    new values.

    Args:
      control_timestep: the new agent's control timestep (in seconds).
      physics_timestep: the new physics simulation timestep (in seconds).

    Raises:
      ValueError: if `control_timestep` is not divisible by `physics_timestep`.
    """
    self._check_root_entity('set_timesteps')
    _check_timesteps_divisible(control_timestep, physics_timestep)
    self.root_entity.mjcf_model.option.timestep = physics_timestep
    self._control_timestep = control_timestep

  @property
  def physics_steps_per_control_step(self):
    """Returns number of physics steps per agent's control step."""
    return _check_timesteps_divisible(
        self.control_timestep, self.physics_timestep)

  def action_spec(self, physics):
    """Returns a `BoundedArray` spec matching the `Physics` actuators.

    BoundedArray.name should contain a tab-separated list of actuator names.
    When overloading this method, non-MuJoCo actuators should be added to the
    top of the list when possible, as a matter of convention.

    Args:
      physics: used to query actuator names in the model.
    """
    names = [physics.model.id2name(i, 'actuator') or str(i)
             for i in range(physics.model.nu)]
    action_spec = mujoco.action_spec(physics)
    return specs.BoundedArray(shape=action_spec.shape,
                              dtype=action_spec.dtype,
                              minimum=action_spec.minimum,
                              maximum=action_spec.maximum,
                              name='\t'.join(names))

  def get_reward_spec(self):
    """Optional method to define non-scalar rewards for a `Task`."""
    return None

  def get_discount_spec(self):
    """Optional method to define non-scalar discounts for a `Task`."""
    return None

  def initialize_episode_mjcf(self, random_state):
    """Modifies the MJCF model of this task before the next episode begins.

    The Environment calls this method and recompiles the physics
    if necessary before calling `initialize_episode`.

    Args:
      random_state: An instance of `np.random.RandomState`.
    """
    pass

  def initialize_episode(self, physics, random_state):
    """Modifies the physics state before the next episode begins.

    The Environment calls this method after `initialize_episode_mjcf`, and also
    after the physics has been recompiled if necessary.

    Args:
      physics: An instance of `control.Physics`.
      random_state: An instance of `np.random.RandomState`.
    """
    pass

  def before_step(self, physics, action, random_state):
    """A callback which is executed before an agent control step.

    The default implementation sets the control signal for the actuators in
    `physics` to be equal to `action`. Subclasses that override this method
    should ensure that the overriding method also sets the control signal before
    returning, either by calling `super(..., self).before_step`, or by setting
    the control signal explicitly (e.g. in order to create a non-trivial mapping
    between `action` and the control signal).

    Args:
      physics: An instance of `control.Physics`.
      action: A NumPy array corresponding to agent actions.
      random_state: An instance of `np.random.RandomState` (unused).
    """
    del random_state  # Unused.
    physics.set_control(action)

  def before_substep(self, physics, action, random_state):
    """A callback which is executed before a simulation step.

    Actuation can be set, or overridden, in this callback.

    Args:
      physics: An instance of `control.Physics`.
      action: A NumPy array corresponding to agent actions.
      random_state: An instance of `np.random.RandomState`.
    """
    pass

  def after_substep(self, physics, random_state):
    """A callback which is executed after a simulation step.

    Args:
      physics: An instance of `control.Physics`.
      random_state: An instance of `np.random.RandomState`.
    """
    pass

  def after_step(self, physics, random_state):
    """A callback which is executed after an agent control step.

    Args:
      physics: An instance of `control.Physics`.
      random_state: An instance of `np.random.RandomState`.
    """
    pass

  @abc.abstractmethod
  def get_reward(self, physics):
    """Calculates the reward signal given the physics state.

    Args:
      physics: A Physics object.

    Returns:
      A float
    """
    raise NotImplementedError

  def should_terminate_episode(self, physics):  # pylint: disable=unused-argument
    """Determines whether the episode should terminate given the physics state.

    Args:
      physics: A Physics object

    Returns:
      A boolean
    """
    return False

  def get_discount(self, physics):  # pylint: disable=unused-argument
    """Calculates the reward discount factor given the physics state.

    Args:
      physics: A Physics object

    Returns:
      A float
    """
    return 1.0


class NullTask(Task):
  """A class that wraps a single `Entity` into a `Task` with no reward."""

  def __init__(self, root_entity):
    self._root_entity = root_entity

  @property
  def root_entity(self):
    return self._root_entity

  def get_reward(self, physics):
    return 0.0



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Tests for `dm_control.manipulation_suite`."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# Internal dependencies.

from absl import flags
from absl.testing import absltest
from absl.testing import parameterized
from dm_control import manipulation
import numpy as np
from six.moves import range


flags.DEFINE_boolean(
    'fix_seed', True,
    'Whether to fix the seed for the environment\'s random number generator. '
    'This the default since it prevents non-deterministic failures, but it may '
    'be useful to allow the seed to vary in some cases, for example when '
    'repeating a test many times in order to detect rare failure events.')

FLAGS = flags.FLAGS

_FIX_SEED = None
_NUM_EPISODES = 5
_NUM_STEPS_PER_EPISODE = 10


def _get_fix_seed():
  global _FIX_SEED
  if _FIX_SEED is None:
    if FLAGS.is_parsed():
      _FIX_SEED = FLAGS.fix_seed
    else:
      _FIX_SEED = FLAGS['fix_seed'].default
  return _FIX_SEED


class DomainTest(parameterized.TestCase):
  """Tests run on all the tasks registered."""

  def _validate_observation(self, observation, observation_spec):
    self.assertEqual(list(observation.keys()), list(observation_spec.keys()))
    for name, array_spec in observation_spec.items():
      array_spec.validate(observation[name])

  def _validate_reward_range(self, reward):
    self.assertIsInstance(reward, float)
    self.assertBetween(reward, 0, 1)

  def _validate_discount(self, discount):
    self.assertIsInstance(discount, float)
    self.assertBetween(discount, 0, 1)

  @parameterized.parameters(*manipulation.ALL)
  def test_task_runs(self, task_name):
    """Tests that the environment runs and is coherent with its specs."""
    seed = 99 if _get_fix_seed() else None
    env = manipulation.load(task_name, seed=seed)
    random_state = np.random.RandomState(seed)

    observation_spec = env.observation_spec()
    action_spec = env.action_spec()
    self.assertTrue(np.all(np.isfinite(action_spec.minimum)))
    self.assertTrue(np.all(np.isfinite(action_spec.maximum)))

    # Run a partial episode, check observations, rewards, discount.
    for _ in range(_NUM_EPISODES):
      time_step = env.reset()
      for _ in range(_NUM_STEPS_PER_EPISODE):
        self._validate_observation(time_step.observation, observation_spec)
        if time_step.first():
          self.assertIsNone(time_step.reward)
          self.assertIsNone(time_step.discount)
        else:
          self._validate_reward_range(time_step.reward)
          self._validate_discount(time_step.discount)
        action = random_state.uniform(action_spec.minimum, action_spec.maximum)
        env.step(action)

if __name__ == '__main__':
  absltest.main()



# Copyright 2019 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Abstract base classes for robot arms and hands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import abc

from dm_control import composer
from dm_control import mjcf
from dm_control.composer import define
from dm_control.composer.observation import observable
from dm_control.mujoco.wrapper import mjbindings
from dm_control.utils import inverse_kinematics
import numpy as np
import six
from six.moves import range
from six.moves import zip


DOWN_QUATERNION = np.array([0., 0.70710678118, 0.70710678118, 0.])

_INVALID_JOINTS_ERROR = (
    'All non-hinge joints must have limits. Model contains the following '
    'non-hinge joints which are unbounded:\n{invalid_str}')


@six.add_metaclass(abc.ABCMeta)
class RobotArm(composer.Robot):
  """The abstract base class for robotic arms."""

  def _build_observables(self):
    return JointsObservables(self)

  @property
  def attachment_site(self):
    return self.wrist_site

  def _get_joint_pos_sampling_bounds(self, physics):
    """Returns lower and upper bounds for sampling arm joint positions.

    Args:
      physics: An `mjcf.Physics` instance.

    Returns:
      A (2, num_joints) numpy array containing (lower, upper) position bounds.
      For hinge joints without limits the bounds are defined as [0, 2pi].

    Raises:
      RuntimeError: If the model contains unlimited joints that are not hinges.
    """
    bound_joints = physics.bind(self.joints)
    limits = np.array(bound_joints.range, copy=True)
    is_hinge = bound_joints.type == mjbindings.enums.mjtJoint.mjJNT_HINGE
    is_limited = bound_joints.limited.astype(np.bool)
    invalid = ~is_hinge & ~is_limited  # All non-hinge joints must have limits.
    if any(invalid):
      invalid_str = '\n'.join(str(self.joints[i]) for i in np.where(invalid)[0])
      raise RuntimeError(_INVALID_JOINTS_ERROR.format(invalid_str=invalid_str))
    # For unlimited hinges we sample positions between 0 and 2pi.
    limits[is_hinge & ~is_limited] = 0., 2*np.pi
    return limits.T

  def randomize_arm_joints(self, physics, random_state):
    """Randomizes the qpos of all arm joints.

    The ranges of qpos values is determined from the MJCF model.

    Args:
      physics: A `mujoco.Physics` instance.
      random_state: An `np.random.RandomState` instance.
    """
    lower, upper = self._get_joint_pos_sampling_bounds(physics)
    physics.bind(self.joints).qpos = random_state.uniform(lower, upper)

  def set_site_to_xpos(self, physics, random_state, site, target_pos,
                       target_quat=None, max_ik_attempts=10):
    """Moves the arm so that a site occurs at the specified location.

    This function runs the inverse kinematics solver to find a configuration
    arm joints for which the pinch site occurs at the specified location in
    Cartesian coordinates.

    Args:
      physics: A `mujoco.Physics` instance.
      random_state: An `np.random.RandomState` instance.
      site: Either a `mjcf.Element` or a string specifying the full name
        of the site whose position is being set.
      target_pos: The desired Cartesian location of the site.
      target_quat: (optional) The desired orientation of the site, expressed
        as a quaternion. If `None`, the default orientation is to point
        vertically downwards.
      max_ik_attempts: (optional) Maximum number of attempts to make at finding
        a solution satisfying `target_pos` and `target_quat`. The joint
        positions will be randomized after each unsuccessful attempt.

    Returns:
      A boolean indicating whether the desired configuration is obtained.

    Raises:
      ValueError: If site is neither a string nor an `mjcf.Element`.
    """
    if isinstance(site, mjcf.Element):
      site_name = site.full_identifier
    elif isinstance(site, str):
      site_name = site
    else:
      raise ValueError('site should either be a string or mjcf.Element: got {}'
                       .format(site))
    if target_quat is None:
      target_quat = DOWN_QUATERNION
    lower, upper = self._get_joint_pos_sampling_bounds(physics)
    arm_joint_names = [joint.full_identifier for joint in self.joints]

    for _ in range(max_ik_attempts):
      result = inverse_kinematics.qpos_from_site_pose(
          physics=physics,
          site_name=site_name,
          target_pos=target_pos,
          target_quat=target_quat,
          joint_names=arm_joint_names,
          rot_weight=2,
          inplace=True)
      success = result.success

      # Canonicalise the angle to [0, 2*pi]
      if success:
        for arm_joint, low, high in zip(self.joints, lower, upper):
          arm_joint_mj = physics.bind(arm_joint)
          while arm_joint_mj.qpos >= high:
            arm_joint_mj.qpos -= 2*np.pi
          while arm_joint_mj.qpos < low:
            arm_joint_mj.qpos += 2*np.pi
            if arm_joint_mj.qpos > high:
              success = False
              break

      # If succeeded or only one attempt, break and do not randomize joints.
      if success or max_ik_attempts <= 1:
        break
      else:
        self.randomize_arm_joints(physics, random_state)

    return success

  @abc.abstractproperty
  def joints(self):
    """Returns the joint elements of the arm."""
    raise NotImplementedError

  @abc.abstractproperty
  def wrist_site(self):
    """Returns the wrist site element of the arm."""
    raise NotImplementedError


class JointsObservables(composer.Observables):
  """Observables common to all robot arms."""

  @define.observable
  def joints_pos(self):
    return observable.MJCFFeature('qpos', self._entity.joints)

  @define.observable
  def joints_vel(self):
    return observable.MJCFFeature('qvel', self._entity.joints)


@six.add_metaclass(abc.ABCMeta)
class RobotHand(composer.Robot):
  """The abstract base class for robotic hands."""

  @abc.abstractmethod
  def set_grasp(self, physics, close_factors):
    """Sets the finger position to the desired positions.

    Args:
      physics: An instance of `mjcf.Physics`.
      close_factors: A number or list of numbers defining the desired grasp
        position of each finger. A value of 0 corresponds to fully opening a
        finger, while a value of 1 corresponds to fully closing it. If a single
        number is specified, the same position is applied to all fingers.
    """

  @abc.abstractproperty
  def tool_center_point(self):
    """Returns the tool center point element of the hand."""



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""String constants for the rendering module."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# Name of the environment variable that selects a renderer platform.
MUJOCO_GL = 'MUJOCO_GL'

# Name of the environment variable that selects a platform for PyOpenGL.
PYOPENGL_PLATFORM = 'PYOPENGL_PLATFORM'

# Renderer platform specifiers.
OSMESA = 'osmesa'
GLFW = 'glfw'
EGL = 'egl'



# Copyright 2017 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Planar Stacker domain."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import collections

from dm_control import mujoco
from dm_control.rl import control
from dm_control.suite import base
from dm_control.suite import common
from dm_control.utils import containers
from dm_control.utils import rewards
from dm_control.utils import xml_tools

from lxml import etree
import numpy as np


_CLOSE = .01    # (Meters) Distance below which a thing is considered close.
_CONTROL_TIMESTEP = .01  # (Seconds)
_TIME_LIMIT = 10  # (Seconds)
_ARM_JOINTS = ['arm_root', 'arm_shoulder', 'arm_elbow', 'arm_wrist',
               'finger', 'fingertip', 'thumb', 'thumbtip']

SUITE = containers.TaggedTasks()


def make_model(n_boxes):
  """Returns a tuple containing the model XML string and a dict of assets."""
  xml_string = common.read_model('stacker.xml')
  parser = etree.XMLParser(remove_blank_text=True)
  mjcf = etree.XML(xml_string, parser)

  # Remove unused boxes
  for b in range(n_boxes, 4):
    box = xml_tools.find_element(mjcf, 'body', 'box' + str(b))
    box.getparent().remove(box)

  return etree.tostring(mjcf, pretty_print=True), common.ASSETS


@SUITE.add('hard')
def stack_2(fully_observable=True, time_limit=_TIME_LIMIT, random=None,
            environment_kwargs=None):
  """Returns stacker task with 2 boxes."""
  n_boxes = 2
  physics = Physics.from_xml_string(*make_model(n_boxes=n_boxes))
  task = Stack(n_boxes=n_boxes,
               fully_observable=fully_observable,
               random=random)
  environment_kwargs = environment_kwargs or {}
  return control.Environment(
      physics, task, control_timestep=_CONTROL_TIMESTEP, time_limit=time_limit,
      **environment_kwargs)


@SUITE.add('hard')
def stack_4(fully_observable=True, time_limit=_TIME_LIMIT, random=None,
            environment_kwargs=None):
  """Returns stacker task with 4 boxes."""
  n_boxes = 4
  physics = Physics.from_xml_string(*make_model(n_boxes=n_boxes))
  task = Stack(n_boxes=n_boxes,
               fully_observable=fully_observable,
               random=random)
  environment_kwargs = environment_kwargs or {}
  return control.Environment(
      physics, task, control_timestep=_CONTROL_TIMESTEP, time_limit=time_limit,
      **environment_kwargs)


class Physics(mujoco.Physics):
  """Physics with additional features for the Planar Manipulator domain."""

  def bounded_joint_pos(self, joint_names):
    """Returns joint positions as (sin, cos) values."""
    joint_pos = self.named.data.qpos[joint_names]
    return np.vstack([np.sin(joint_pos), np.cos(joint_pos)]).T

  def joint_vel(self, joint_names):
    """Returns joint velocities."""
    return self.named.data.qvel[joint_names]

  def body_2d_pose(self, body_names, orientation=True):
    """Returns positions and/or orientations of bodies."""
    if not isinstance(body_names, str):
      body_names = np.array(body_names).reshape(-1, 1)  # Broadcast indices.
    pos = self.named.data.xpos[body_names, ['x', 'z']]
    if orientation:
      ori = self.named.data.xquat[body_names, ['qw', 'qy']]
      return np.hstack([pos, ori])
    else:
      return pos

  def touch(self):
    return np.log1p(self.data.sensordata)

  def site_distance(self, site1, site2):
    site1_to_site2 = np.diff(self.named.data.site_xpos[[site2, site1]], axis=0)
    return np.linalg.norm(site1_to_site2)


class Stack(base.Task):
  """A Stack `Task`: stack the boxes."""

  def __init__(self, n_boxes, fully_observable, random=None):
    """Initialize an instance of the `Stack` task.

    Args:
      n_boxes: An `int`, number of boxes to stack.
      fully_observable: A `bool`, whether the observation should contain the
        positions and velocities of the boxes and the location of the target.
      random: Optional, either a `numpy.random.RandomState` instance, an
        integer seed for creating a new `RandomState`, or None to select a seed
        automatically (default).
    """
    self._n_boxes = n_boxes
    self._box_names = ['box' + str(b) for b in range(n_boxes)]
    self._box_joint_names = []
    for name in self._box_names:
      for dim in 'xyz':
        self._box_joint_names.append('_'.join([name, dim]))
    self._fully_observable = fully_observable
    super(Stack, self).__init__(random=random)

  def initialize_episode(self, physics):
    """Sets the state of the environment at the start of each episode."""
    # Local aliases
    randint = self.random.randint
    uniform = self.random.uniform
    model = physics.named.model
    data = physics.named.data

    # Find a collision-free random initial configuration.
    penetrating = True
    while penetrating:

      # Randomise angles of arm joints.
      is_limited = model.jnt_limited[_ARM_JOINTS].astype(np.bool)
      joint_range = model.jnt_range[_ARM_JOINTS]
      lower_limits = np.where(is_limited, joint_range[:, 0], -np.pi)
      upper_limits = np.where(is_limited, joint_range[:, 1], np.pi)
      angles = uniform(lower_limits, upper_limits)
      data.qpos[_ARM_JOINTS] = angles

      # Symmetrize hand.
      data.qpos['finger'] = data.qpos['thumb']

      # Randomise target location.
      target_height = 2*randint(self._n_boxes) + 1
      box_size = model.geom_size['target', 0]
      model.body_pos['target', 'z'] = box_size * target_height
      model.body_pos['target', 'x'] = uniform(-.37, .37)

      # Randomise box locations.
      for name in self._box_names:
        data.qpos[name + '_x'] = uniform(.1, .3)
        data.qpos[name + '_z'] = uniform(0, .7)
        data.qpos[name + '_y'] = uniform(0, 2*np.pi)

      # Check for collisions.
      physics.after_reset()
      penetrating = physics.data.ncon > 0

    super(Stack, self).initialize_episode(physics)

  def get_observation(self, physics):
    """Returns either features or only sensors (to be used with pixels)."""
    obs = collections.OrderedDict()
    obs['arm_pos'] = physics.bounded_joint_pos(_ARM_JOINTS)
    obs['arm_vel'] = physics.joint_vel(_ARM_JOINTS)
    obs['touch'] = physics.touch()
    if self._fully_observable:
      obs['hand_pos'] = physics.body_2d_pose('hand')
      obs['box_pos'] = physics.body_2d_pose(self._box_names)
      obs['box_vel'] = physics.joint_vel(self._box_joint_names)
      obs['target_pos'] = physics.body_2d_pose('target', orientation=False)
    return obs

  def get_reward(self, physics):
    """Returns a reward to the agent."""
    box_size = physics.named.model.geom_size['target', 0]
    min_box_to_target_distance = min(physics.site_distance(name, 'target')
                                     for name in self._box_names)
    box_is_close = rewards.tolerance(min_box_to_target_distance,
                                     margin=2*box_size)
    hand_to_target_distance = physics.site_distance('grasp', 'target')
    hand_is_far = rewards.tolerance(hand_to_target_distance,
                                    bounds=(.1, float('inf')),
                                    margin=_CLOSE)
    return box_is_close * hand_is_far



# Copyright 2018-2019 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Runtime tests."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import collections
from absl.testing import absltest
from absl.testing import parameterized
from dm_control.viewer import runtime
import dm_env
from dm_env import specs
import mock
import numpy as np
import six
from six.moves import zip


class RuntimeStateMachineTest(parameterized.TestCase):

  def setUp(self):
    super(RuntimeStateMachineTest, self).setUp()
    env = mock.MagicMock()
    env.action_spec.return_value = specs.BoundedArray((1,), np.float64, -1, 1)
    self.runtime = runtime.Runtime(env, mock.MagicMock())
    self.runtime._start = mock.MagicMock()
    self.runtime.get_time = mock.MagicMock()
    self.runtime.get_time.return_value = 0
    self.runtime._step_simulation = mock.MagicMock(return_value=False)

  def test_initial_state(self):
    self.assertEqual(self.runtime._state, runtime.State.START)

  def test_successful_starting(self):
    self.runtime._start.return_value = True
    self.runtime._state = runtime.State.START
    self.runtime.tick(0, False)
    self.assertEqual(self.runtime._state, runtime.State.RUNNING)
    self.runtime._start.assert_called_once()
    self.runtime._step_simulation.assert_called_once()

  def test_failure_during_start(self):
    self.runtime._start.return_value = False
    self.runtime._state = runtime.State.START
    self.runtime.tick(0, False)
    self.assertEqual(self.runtime._state, runtime.State.STOPPED)
    self.runtime._start.assert_called_once()
    self.runtime._step_simulation.assert_not_called()

  def test_restarting(self):
    self.runtime._state = runtime.State.RUNNING
    self.runtime.restart()
    self.runtime.tick(0, False)
    self.assertEqual(self.runtime._state, runtime.State.RUNNING)
    self.runtime._start.assert_called_once()
    self.runtime._step_simulation.assert_called_once()

  def test_running(self):
    self.runtime._state = runtime.State.RUNNING
    self.runtime.tick(0, False)
    self.assertEqual(self.runtime._state, runtime.State.RUNNING)
    self.runtime._step_simulation.assert_called_once()

  def test_ending_a_running_episode(self):
    self.runtime._state = runtime.State.RUNNING
    self.runtime._step_simulation.return_value = True
    self.runtime.tick(0, False)
    self.assertEqual(self.runtime._state, runtime.State.STOPPED)
    self.runtime._step_simulation.assert_called_once()

  def test_calling_stop_has_immediate_effect_on_state(self):
    self.runtime.stop()
    self.assertEqual(self.runtime._state, runtime.State.STOPPED)

  @parameterized.parameters(runtime.State.RUNNING,
                            runtime.State.RESTARTING,)
  def test_states_affected_by_stop(self, state):
    self.runtime._state = state
    self.runtime.stop()
    self.assertEqual(self.runtime._state, runtime.State.STOPPED)

  def test_notifying_listeners_about_successful_start(self):
    callback = mock.MagicMock()
    self.runtime.on_episode_begin += [callback]
    self.runtime._start.return_value = True
    self.runtime._state = runtime.State.START
    self.runtime.tick(0, False)
    callback.assert_called_once()

  def test_listeners_not_notified_when_start_fails(self):
    callback = mock.MagicMock()
    self.runtime.on_episode_begin += [callback]
    self.runtime._start.return_value = False
    self.runtime._state = runtime.State.START
    self.runtime.tick(0, False)
    callback.assert_not_called()


class RuntimeSingleStepTest(parameterized.TestCase):

  def setUp(self):
    super(RuntimeSingleStepTest, self).setUp()
    env = mock.MagicMock(spec=dm_env.Environment)
    env.action_spec.return_value = specs.BoundedArray((1,), np.float64, -1, 1)
    self.runtime = runtime.Runtime(env, mock.MagicMock())
    self.runtime._step = mock.MagicMock()
    self.runtime._step.return_value = False

  def test_when_running(self):
    self.runtime._state = runtime.State.RUNNING
    self.runtime.single_step()
    self.assertEqual(self.runtime._state, runtime.State.RUNNING)
    self.runtime._step.assert_called_once()

  def test_ending_episode(self):
    self.runtime._state = runtime.State.RUNNING
    self.runtime._step.return_value = True
    self.runtime.single_step()
    self.assertEqual(self.runtime._state, runtime.State.STOP)
    self.runtime._step.assert_called_once()

  @parameterized.parameters(runtime.State.START,
                            runtime.State.STOP,
                            runtime.State.STOPPED,
                            runtime.State.RESTARTING)
  def test_runs_only_in_running_state(self, state):
    self.runtime._state = state
    self.runtime.single_step()
    self.assertEqual(self.runtime._state, state)
    self.assertEqual(0, self.runtime._step.call_count)


class RuntimeTest(absltest.TestCase):

  def setUp(self):
    super(RuntimeTest, self).setUp()
    env = mock.MagicMock(spec=dm_env.Environment)
    env.action_spec.return_value = specs.BoundedArray((1,), np.float64, -1, 1)
    self.runtime = runtime.Runtime(env, mock.MagicMock())
    self.runtime._step_paused = mock.MagicMock()
    self.runtime._step = mock.MagicMock(return_value=True)
    self.runtime.get_time = mock.MagicMock(return_value=0)

    self.time_step = 1e-2

  def set_loop(self, num_iterations, finish_after=0):
    finish_after = finish_after or num_iterations + 1
    self.delta_time = self.time_step / float(num_iterations)
    self.time = 0
    self.iteration = 0
    def fakeget_time():
      return self.time
    def fake_step():
      self.time += self.delta_time
      self.iteration += 1
      return self.iteration >= finish_after

    self.runtime._step = mock.MagicMock(side_effect=fake_step)
    self.runtime.get_time = mock.MagicMock(side_effect=fakeget_time)

  def test_num_step_calls(self):
    expected_call_count = 5
    self.set_loop(num_iterations=expected_call_count)
    finished = self.runtime._step_simulation(self.time_step, False)
    self.assertFalse(finished)
    self.assertEqual(expected_call_count, self.runtime._step.call_count)

  def test_finishing_if_episode_ends(self):
    num_iterations = 5
    finish_after = 2
    self.set_loop(num_iterations=num_iterations, finish_after=finish_after)
    finished = self.runtime._step_simulation(self.time_step, False)
    self.assertTrue(finished)
    self.assertEqual(finish_after, self.runtime._step.call_count)

  def test_stepping_paused(self):
    self.runtime._step_simulation(0, True)
    self.runtime._step_paused.assert_called_once()
    self.assertEqual(0, self.runtime._step.call_count)

  def test_physics_step_takes_less_time_than_tick(self):
    self.physics_time_step = runtime._DEFAULT_MAX_SIM_STEP * 0.5
    self.physics_time = 0.0
    def mock_get_time():
      return self.physics_time
    def mock_step():
      self.physics_time += self.physics_time_step
    self.runtime._step = mock.MagicMock(side_effect=mock_step)
    self.runtime.get_time = mock.MagicMock(side_effect=mock_get_time)
    self.runtime._step_simulation(
        time_elapsed=runtime._DEFAULT_MAX_SIM_STEP, paused=False)
    self.assertEqual(2, self.runtime._step.call_count)

  def test_physics_step_takes_more_time_than_tick(self):
    self.physics_time_step = runtime._DEFAULT_MAX_SIM_STEP * 2
    self.physics_time = 0.0
    def mock_get_time():
      return self.physics_time
    def mock_step():
      self.physics_time += self.physics_time_step
    self.runtime._step = mock.MagicMock(side_effect=mock_step)
    self.runtime.get_time = mock.MagicMock(side_effect=mock_get_time)

    # Simulates after the first frame
    self.runtime._step_simulation(
        time_elapsed=runtime._DEFAULT_MAX_SIM_STEP, paused=False)
    self.assertEqual(1, self.runtime._step.call_count)
    self.runtime._step.reset_mock()

    # Then pauses for one frame to let the internal timer catch up with the
    # simulation timer.
    self.runtime._step_simulation(
        time_elapsed=runtime._DEFAULT_MAX_SIM_STEP, paused=False)
    self.assertEqual(0, self.runtime._step.call_count)

    # Resumes simulation on the subsequent frame.
    self.runtime._step_simulation(
        time_elapsed=runtime._DEFAULT_MAX_SIM_STEP, paused=False)
    self.assertEqual(1, self.runtime._step.call_count)

  def test_updating_tracked_time_during_start(self):
    invalid_time = 20
    self.runtime.get_time = mock.MagicMock(return_value=invalid_time)

    valid_time = 2
    def mock_start():
      self.runtime.get_time = mock.MagicMock(return_value=valid_time)
      return True

    self.runtime._start = mock.MagicMock(side_effect=mock_start)
    self.runtime._step_simulation = mock.MagicMock()

    self.runtime.tick(time_elapsed=runtime._DEFAULT_MAX_SIM_STEP, paused=False)
    self.assertEqual(valid_time, self.runtime._tracked_simulation_time)

  def test_error_logger_forward_errors_to_listeners(self):
    callback = mock.MagicMock()
    self.runtime.on_error += [callback]
    with self.runtime._error_logger:
      raise Exception('error message')
    callback.assert_called_once()


class EnvironmentRuntimeTest(parameterized.TestCase):

  def setUp(self):
    super(EnvironmentRuntimeTest, self).setUp()
    self.observation = mock.MagicMock()
    self.env = mock.MagicMock(spec=dm_env.Environment)
    self.env.physics = mock.MagicMock()
    self.env.step = mock.MagicMock()
    self.env.action_spec.return_value = specs.BoundedArray(
        (1,), np.float64, -1, 1)
    self.policy = mock.MagicMock()
    self.actions = mock.MagicMock()
    self.runtime = runtime.Runtime(self.env, self.policy)

  def test_start(self):
    with mock.patch(runtime.__name__ + '.mjlib'):
      result = self.runtime._start()
      self.assertTrue(result)
      self.env.reset.assert_called_once()
      self.policy.assert_not_called()

  def test_step_with_policy(self):
    time_step = mock.Mock(spec=dm_env.TimeStep)
    self.runtime._time_step = time_step
    self.runtime._step()
    self.policy.assert_called_once_with(time_step)
    self.env.step.assert_called_once_with(self.policy.return_value)

  def test_step_without_policy(self):
    with mock.patch(
        runtime.__name__ + '._get_default_action') as mock_get_default_action:
      this_runtime = runtime.Runtime(environment=self.env, policy=None)
    this_runtime._step()
    self.env.step.assert_called_once_with(mock_get_default_action.return_value)

  def test_stepping_paused(self):
    with mock.patch(runtime.__name__ + '.mjlib') as mjlib:
      self.runtime._step_paused()
      mjlib.mj_forward.assert_called_once()

  def test_get_time(self):
    expected_time = 20
    self.env.physics = mock.MagicMock()
    self.env.physics.data = mock.MagicMock()
    self.env.physics.data.time = expected_time
    self.assertEqual(expected_time, self.runtime.get_time())

  def test_tracking_physics_instance_changes(self):
    callback = mock.MagicMock()
    self.runtime.on_physics_changed += [callback]

    def begin_episode_and_reload_physics():
      self.env.physics.data.ptr = mock.MagicMock()
    self.env.reset.side_effect = begin_episode_and_reload_physics

    self.runtime._start()
    callback.assert_called_once_with()

  def test_tracking_physics_instance_that_doesnt_change(self):
    callback = mock.MagicMock()
    self.runtime.on_physics_changed += [callback]

    self.runtime._start()
    callback.assert_not_called()

  def test_exception_thrown_during_start(self):
    def raise_exception(*unused_args, **unused_kwargs):
      raise Exception('test error message')
    self.runtime._env.reset.side_effect = raise_exception
    result = self.runtime._start()
    self.assertFalse(result)

  def test_exception_thrown_during_step(self):
    def raise_exception(*unused_args, **unused_kwargs):
      raise Exception('test error message')
    self.runtime._env.step.side_effect = raise_exception
    finished = self.runtime._step()
    self.assertTrue(finished)


class DefaultActionFromSpecTest(parameterized.TestCase):

  def assertNestedArraysEqual(self, expected, actual):
    """Asserts that two potentially nested structures of arrays are equal."""
    self.assertIs(type(actual), type(expected))
    if isinstance(expected, (list, tuple)):
      self.assertIsInstance(actual, (list, tuple))
      self.assertLen(actual, len(expected))
      for expected_item, actual_item in zip(expected, actual):
        self.assertNestedArraysEqual(expected_item, actual_item)
    elif isinstance(expected, collections.MutableMapping):
      keys_type = list if isinstance(expected, collections.OrderedDict) else set
      self.assertEqual(keys_type(actual.keys()), keys_type(expected.keys()))
      for key, expected_value in six.iteritems(expected):
        self.assertNestedArraysEqual(actual[key], expected_value)
    else:
      np.testing.assert_array_equal(expected, actual)

  _SHAPE = (2,)
  _DTYPE = np.float64
  _ACTION = np.zeros(_SHAPE)
  _ACTION_SPEC = specs.BoundedArray(_SHAPE, np.float64, -1, 1)

  @parameterized.named_parameters(
      ('single_array', _ACTION_SPEC, _ACTION),
      ('tuple', (_ACTION_SPEC, _ACTION_SPEC), (_ACTION, _ACTION)),
      ('list', [_ACTION_SPEC, _ACTION_SPEC], (_ACTION, _ACTION)),
      ('dict',
       {'a': _ACTION_SPEC, 'b': _ACTION_SPEC},
       {'a': _ACTION, 'b': _ACTION}),
      ('OrderedDict',
       collections.OrderedDict([('a', _ACTION_SPEC), ('b', _ACTION_SPEC)]),
       collections.OrderedDict([('a', _ACTION), ('b', _ACTION)])),
      )
  def test_action_structure(self, action_spec, expected_action):
    self.assertNestedArraysEqual(expected_action,
                                 runtime._get_default_action(action_spec))

  def test_ordered_dict_action_structure_with_bad_ordering(self):
    reversed_spec = collections.OrderedDict([('a', self._ACTION_SPEC),
                                             ('b', self._ACTION_SPEC)])
    expected_action = collections.OrderedDict([('b', self._ACTION),
                                               ('a', self._ACTION)])
    with six.assertRaisesRegex(self, AssertionError,
                               r"Lists differ: \['a', 'b'\] != \['b', 'a'\]"):
      self.assertNestedArraysEqual(expected_action,
                                   runtime._get_default_action(reversed_spec))

  @parameterized.named_parameters(
      ('closed',
       specs.BoundedArray(_SHAPE, _DTYPE, minimum=1., maximum=2.),
       np.full(_SHAPE, fill_value=1.5, dtype=_DTYPE)),
      ('left_open',
       specs.BoundedArray(_SHAPE, _DTYPE, minimum=-np.inf, maximum=2.),
       np.full(_SHAPE, fill_value=2., dtype=_DTYPE)),
      ('right_open',
       specs.BoundedArray(_SHAPE, _DTYPE, minimum=1., maximum=np.inf),
       np.full(_SHAPE, fill_value=1., dtype=_DTYPE)),
      ('unbounded',
       specs.BoundedArray(_SHAPE, _DTYPE, minimum=-np.inf, maximum=np.inf),
       np.full(_SHAPE, fill_value=0., dtype=_DTYPE)))
  def test_action_spec_interval(self, action_spec, expected_action):
    self.assertNestedArraysEqual(expected_action,
                                 runtime._get_default_action(action_spec))


if __name__ == '__main__':
  absltest.main()



# Copyright 2017 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Procedurally generated Swimmer domain."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import collections

from dm_control import mujoco
from dm_control.rl import control
from dm_control.suite import base
from dm_control.suite import common
from dm_control.suite.utils import randomizers
from dm_control.utils import containers
from dm_control.utils import rewards
from lxml import etree
import numpy as np
from six.moves import range

_DEFAULT_TIME_LIMIT = 30
_CONTROL_TIMESTEP = .03  # (Seconds)

SUITE = containers.TaggedTasks()


def get_model_and_assets(n_joints):
  """Returns a tuple containing the model XML string and a dict of assets.

  Args:
    n_joints: An integer specifying the number of joints in the swimmer.

  Returns:
    A tuple `(model_xml_string, assets)`, where `assets` is a dict consisting of
    `{filename: contents_string}` pairs.
  """
  return _make_model(n_joints), common.ASSETS


@SUITE.add('benchmarking')
def swimmer6(time_limit=_DEFAULT_TIME_LIMIT, random=None,
             environment_kwargs=None):
  """Returns a 6-link swimmer."""
  return _make_swimmer(6, time_limit, random=random,
                       environment_kwargs=environment_kwargs)


@SUITE.add('benchmarking')
def swimmer15(time_limit=_DEFAULT_TIME_LIMIT, random=None,
              environment_kwargs=None):
  """Returns a 15-link swimmer."""
  return _make_swimmer(15, time_limit, random=random,
                       environment_kwargs=environment_kwargs)


def swimmer(n_links=3, time_limit=_DEFAULT_TIME_LIMIT,
            random=None, environment_kwargs=None):
  """Returns a swimmer with n links."""
  return _make_swimmer(n_links, time_limit, random=random,
                       environment_kwargs=environment_kwargs)


def _make_swimmer(n_joints, time_limit=_DEFAULT_TIME_LIMIT, random=None,
                  environment_kwargs=None):
  """Returns a swimmer control environment."""
  model_string, assets = get_model_and_assets(n_joints)
  physics = Physics.from_xml_string(model_string, assets=assets)
  task = Swimmer(random=random)
  environment_kwargs = environment_kwargs or {}
  return control.Environment(
      physics, task, time_limit=time_limit, control_timestep=_CONTROL_TIMESTEP,
      **environment_kwargs)


def _make_model(n_bodies):
  """Generates an xml string defining a swimmer with `n_bodies` bodies."""
  if n_bodies < 3:
    raise ValueError('At least 3 bodies required. Received {}'.format(n_bodies))
  mjcf = etree.fromstring(common.read_model('swimmer.xml'))
  head_body = mjcf.find('./worldbody/body')
  actuator = etree.SubElement(mjcf, 'actuator')
  sensor = etree.SubElement(mjcf, 'sensor')

  parent = head_body
  for body_index in range(n_bodies - 1):
    site_name = 'site_{}'.format(body_index)
    child = _make_body(body_index=body_index)
    child.append(etree.Element('site', name=site_name))
    joint_name = 'joint_{}'.format(body_index)
    joint_limit = 360.0/n_bodies
    joint_range = '{} {}'.format(-joint_limit, joint_limit)
    child.append(etree.Element('joint', {'name': joint_name,
                                         'range': joint_range}))
    motor_name = 'motor_{}'.format(body_index)
    actuator.append(etree.Element('motor', name=motor_name, joint=joint_name))
    velocimeter_name = 'velocimeter_{}'.format(body_index)
    sensor.append(etree.Element('velocimeter', name=velocimeter_name,
                                site=site_name))
    gyro_name = 'gyro_{}'.format(body_index)
    sensor.append(etree.Element('gyro', name=gyro_name, site=site_name))
    parent.append(child)
    parent = child

  # Move tracking cameras further away from the swimmer according to its length.
  cameras = mjcf.findall('./worldbody/body/camera')
  scale = n_bodies / 6.0
  for cam in cameras:
    if cam.get('mode') == 'trackcom':
      old_pos = cam.get('pos').split(' ')
      new_pos = ' '.join([str(float(dim) * scale) for dim in old_pos])
      cam.set('pos', new_pos)

  return etree.tostring(mjcf, pretty_print=True)


def _make_body(body_index):
  """Generates an xml string defining a single physical body."""
  body_name = 'segment_{}'.format(body_index)
  visual_name = 'visual_{}'.format(body_index)
  inertial_name = 'inertial_{}'.format(body_index)
  body = etree.Element('body', name=body_name)
  body.set('pos', '0 .1 0')
  etree.SubElement(body, 'geom', {'class': 'visual', 'name': visual_name})
  etree.SubElement(body, 'geom', {'class': 'inertial', 'name': inertial_name})
  return body


class Physics(mujoco.Physics):
  """Physics simulation with additional features for the swimmer domain."""

  def nose_to_target(self):
    """Returns a vector from nose to target in local coordinate of the head."""
    nose_to_target = (self.named.data.geom_xpos['target'] -
                      self.named.data.geom_xpos['nose'])
    head_orientation = self.named.data.xmat['head'].reshape(3, 3)
    return nose_to_target.dot(head_orientation)[:2]

  def nose_to_target_dist(self):
    """Returns the distance from the nose to the target."""
    return np.linalg.norm(self.nose_to_target())

  def body_velocities(self):
    """Returns local body velocities: x,y linear, z rotational."""
    xvel_local = self.data.sensordata[12:].reshape((-1, 6))
    vx_vy_wz = [0, 1, 5]  # Indices for linear x,y vels and rotational z vel.
    return xvel_local[:, vx_vy_wz].ravel()

  def joints(self):
    """Returns all internal joint angles (excluding root joints)."""
    return self.data.qpos[3:].copy()


class Swimmer(base.Task):
  """A swimmer `Task` to reach the target or just swim."""

  def __init__(self, random=None):
    """Initializes an instance of `Swimmer`.

    Args:
      random: Optional, either a `numpy.random.RandomState` instance, an
        integer seed for creating a new `RandomState`, or None to select a seed
        automatically (default).
    """
    super(Swimmer, self).__init__(random=random)

  def initialize_episode(self, physics):
    """Sets the state of the environment at the start of each episode.

    Initializes the swimmer orientation to [-pi, pi) and the relative joint
    angle of each joint uniformly within its range.

    Args:
      physics: An instance of `Physics`.
    """
    # Random joint angles:
    randomizers.randomize_limited_and_rotational_joints(physics, self.random)
    # Random target position.
    close_target = self.random.rand() < .2  # Probability of a close target.
    target_box = .3 if close_target else 2
    xpos, ypos = self.random.uniform(-target_box, target_box, size=2)
    physics.named.model.geom_pos['target', 'x'] = xpos
    physics.named.model.geom_pos['target', 'y'] = ypos
    physics.named.model.light_pos['target_light', 'x'] = xpos
    physics.named.model.light_pos['target_light', 'y'] = ypos

    super(Swimmer, self).initialize_episode(physics)

  def get_observation(self, physics):
    """Returns an observation of joint angles, body velocities and target."""
    obs = collections.OrderedDict()
    obs['joints'] = physics.joints()
    obs['to_target'] = physics.nose_to_target()
    obs['body_velocities'] = physics.body_velocities()
    return obs

  def get_reward(self, physics):
    """Returns a smooth reward."""
    target_size = physics.named.model.geom_size['target', 0]
    return rewards.tolerance(physics.nose_to_target_dist(),
                             bounds=(0, target_size),
                             margin=5*target_size,
                             sigmoid='long_tail')



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Magic constants used within `dm_control.mjcf`."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

PREFIX_SEPARATOR = '/'
PREFIX_SEPARATOR_ESCAPE = '\\'

# Used to disambiguate namespaces between attachment frames.
NAMESPACE_SEPARATOR = '@'

# Magic attribute names
BASEPATH = 'basepath'
CHILDCLASS = 'childclass'
CLASS = 'class'
DEFAULT = 'default'
DCLASS = 'dclass'

# Magic tags
ACTUATOR = 'actuator'
BODY = 'body'
DEFAULT = 'default'
MESH = 'mesh'
SITE = 'site'
TENDON = 'tendon'
WORLDBODY = 'worldbody'

MJDATA_TRIGGERS_DIRTY = [
    'qpos', 'qvel', 'act', 'ctrl', 'qfrc_applied', 'xfrc_applied']
MJMODEL_DOESNT_TRIGGER_DIRTY = [
    'rgba', 'matid', 'emission', 'specular', 'shininess', 'reflectance']

# When writing into `model.{body,geom,site}_{pos,quat}` we must ensure that the
# corresponding rows in `model.{body,geom,site}_sameframe` are set to zero,
# otherwise MuJoCo will use the body or inertial frame instead of our modified
# pos/quat values. We must do the same for `body_{ipos,iquat}` and
# `body_simple`.
MJMODEL_DISABLE_ON_WRITE = {
    # Field name in MjModel: (attribute names of Binding instance to be zeroed)
    'body_pos': ('sameframe',),
    'body_quat': ('sameframe',),
    'geom_pos': ('sameframe',),
    'geom_quat': ('sameframe',),
    'site_pos': ('sameframe',),
    'site_quat': ('sameframe',),
    'body_ipos': ('simple', 'sameframe'),
    'body_iquat': ('simple', 'sameframe'),
}

# This is the actual upper limit on VFS filename length, despite what it says
# in the header file (100) or the error message (99).
MAX_VFS_FILENAME_LENGTH = 98

# The prefix used in the schema to denote reference_namespace that are defined
# via another attribute.
INDIRECT_REFERENCE_NAMESPACE_PREFIX = 'attrib:'



# Copyright 2017 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Point-mass domain."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import collections

from dm_control import mujoco
from dm_control.rl import control
from dm_control.suite import base
from dm_control.suite import common
from dm_control.suite.utils import randomizers
from dm_control.utils import containers
from dm_control.utils import rewards
import numpy as np

_DEFAULT_TIME_LIMIT = 20
SUITE = containers.TaggedTasks()


def get_model_and_assets():
  """Returns a tuple containing the model XML string and a dict of assets."""
  return common.read_model('point_mass.xml'), common.ASSETS


@SUITE.add('benchmarking', 'easy')
def easy(time_limit=_DEFAULT_TIME_LIMIT, random=None, environment_kwargs=None):
  """Returns the easy point_mass task."""
  physics = Physics.from_xml_string(*get_model_and_assets())
  task = PointMass(randomize_gains=False, random=random)
  environment_kwargs = environment_kwargs or {}
  return control.Environment(
      physics, task, time_limit=time_limit, **environment_kwargs)


@SUITE.add()
def hard(time_limit=_DEFAULT_TIME_LIMIT, random=None, environment_kwargs=None):
  """Returns the hard point_mass task."""
  physics = Physics.from_xml_string(*get_model_and_assets())
  task = PointMass(randomize_gains=True, random=random)
  environment_kwargs = environment_kwargs or {}
  return control.Environment(
      physics, task, time_limit=time_limit, **environment_kwargs)


class Physics(mujoco.Physics):
  """physics for the point_mass domain."""

  def mass_to_target(self):
    """Returns the vector from mass to target in global coordinate."""
    return (self.named.data.geom_xpos['target'] -
            self.named.data.geom_xpos['pointmass'])

  def mass_to_target_dist(self):
    """Returns the distance from mass to the target."""
    return np.linalg.norm(self.mass_to_target())


class PointMass(base.Task):
  """A point_mass `Task` to reach target with smooth reward."""

  def __init__(self, randomize_gains, random=None):
    """Initialize an instance of `PointMass`.

    Args:
      randomize_gains: A `bool`, whether to randomize the actuator gains.
      random: Optional, either a `numpy.random.RandomState` instance, an
        integer seed for creating a new `RandomState`, or None to select a seed
        automatically (default).
    """
    self._randomize_gains = randomize_gains
    super(PointMass, self).__init__(random=random)

  def initialize_episode(self, physics):
    """Sets the state of the environment at the start of each episode.

       If _randomize_gains is True, the relationship between the controls and
       the joints is randomized, so that each control actuates a random linear
       combination of joints.

    Args:
      physics: An instance of `mujoco.Physics`.
    """
    randomizers.randomize_limited_and_rotational_joints(physics, self.random)
    if self._randomize_gains:
      dir1 = self.random.randn(2)
      dir1 /= np.linalg.norm(dir1)
      # Find another actuation direction that is not 'too parallel' to dir1.
      parallel = True
      while parallel:
        dir2 = self.random.randn(2)
        dir2 /= np.linalg.norm(dir2)
        parallel = abs(np.dot(dir1, dir2)) > 0.9
      physics.model.wrap_prm[[0, 1]] = dir1
      physics.model.wrap_prm[[2, 3]] = dir2
    super(PointMass, self).initialize_episode(physics)

  def get_observation(self, physics):
    """Returns an observation of the state."""
    obs = collections.OrderedDict()
    obs['position'] = physics.position()
    obs['velocity'] = physics.velocity()
    return obs

  def get_reward(self, physics):
    """Returns a reward to the agent."""
    target_size = physics.named.model.geom_size['target', 0]
    near_target = rewards.tolerance(physics.mass_to_target_dist(),
                                    bounds=(0, target_size), margin=target_size)
    control_reward = rewards.tolerance(physics.control(), margin=1,
                                       value_at_margin=0,
                                       sigmoid='quadratic').mean()
    small_control = (control_reward + 4) / 5
    return near_target * small_control



# Copyright 2017 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Tests for utils.xml_tools."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# Internal dependencies.
from absl.testing import absltest
from dm_control.utils import xml_tools
import lxml
import six

etree = lxml.etree


class XmlHelperTest(absltest.TestCase):

  def test_nested(self):
    element = etree.Element('inserted')
    xml_tools.nested_element(element, depth=2)
    level_1 = element.find('inserted')
    self.assertIsNotNone(level_1)
    level_2 = level_1.find('inserted')
    self.assertIsNotNone(level_2)

  def test_tostring(self):
    xml_str = """
    <root>
      <head>
        <content></content>
      </head>
    </root>"""
    tree = xml_tools.parse(six.StringIO(xml_str))
    self.assertEqual(b'<root>\n  <head>\n    <content/>\n  </head>\n</root>\n',
                     etree.tostring(tree, pretty_print=True))

  def test_find_element(self):
    xml_str = """
    <root>
      <option name='option_name'>
        <content></content>
      </option>
      <world name='world_name'>
        <geom name='geom_name'/>
      </world>
    </root>"""
    tree = xml_tools.parse(six.StringIO(xml_str))
    world = xml_tools.find_element(root=tree, tag='world', name='world_name')
    self.assertEqual(world.tag, 'world')
    self.assertEqual(world.attrib['name'], 'world_name')

    geom = xml_tools.find_element(root=tree, tag='geom', name='geom_name')
    self.assertEqual(geom.tag, 'geom')
    self.assertEqual(geom.attrib['name'], 'geom_name')

    with six.assertRaisesRegex(self, ValueError, 'Element with tag'):
      xml_tools.find_element(root=tree, tag='does_not_exist', name='name')

    with six.assertRaisesRegex(self, ValueError, 'Element with tag'):
      xml_tools.find_element(root=tree, tag='world', name='does_not_exist')


if __name__ == '__main__':
  absltest.main()



# Copyright 2019 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Initializers for the locomotion walkers."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import abc
import numpy as np
import six


@six.add_metaclass(abc.ABCMeta)
class WalkerInitializer(object):
  """The abstract base class for a walker initializer."""

  @abc.abstractmethod
  def initialize_pose(self, physics, walker, random_state):
    raise NotImplementedError


class UprightInitializer(WalkerInitializer):
  """An initializer that uses the walker-declared upright pose."""

  def initialize_pose(self, physics, walker, random_state):
    all_joints_binding = physics.bind(walker.mjcf_model.find_all('joint'))
    qpos, xpos, xquat = walker.upright_pose
    if qpos is None:
      all_joints_binding.qpos = all_joints_binding.qpos0
    else:
      all_joints_binding.qpos = qpos
    walker.set_pose(physics, position=xpos, quaternion=xquat)
    walker.set_velocity(
        physics, velocity=np.zeros(3), angular_velocity=np.zeros(3))


class RandomlySampledInitializer(WalkerInitializer):
  """An initializer that random selects between many initializers."""

  def __init__(self, initializers):
    self._initializers = initializers
    self.num_initializers = len(initializers)

  def initialize_pose(self, physics, walker, random_state):
    random_initalizer_idx = np.random.randint(0, self.num_initializers)
    self._initializers[random_initalizer_idx].initialize_pose(
        physics, walker, random_state)


class NoOpInitializer(WalkerInitializer):
  """An initializer that does nothing."""

  def initialize_pose(self, physics, walker, random_state):
    pass






# Copyright 2020 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""A non-colliding sphere that is activated through touch."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from dm_control import composer
from dm_control import mjcf


class TargetSphere(composer.Entity):
  """A non-colliding sphere that is activated through touch.

  Once the target has been reached, it remains in the "activated" state
  for the remainder of the current episode.

  The target is automatically reset to "not activated" state at episode
  initialization time.
  """

  def _build(self,
             radius=0.6,
             height_above_ground=1,
             rgb1=(0, 0.4, 0),
             rgb2=(0, 0.7, 0),
             specific_collision_geom_ids=None,
             name='target'):
    """Builds this target sphere.

    Args:
      radius: The radius (in meters) of this target sphere.
      height_above_ground: The height (in meters) of this target above ground.
      rgb1: A sequence of three floating point values between 0.0 and 1.0
        (inclusive) representing the color of the first element in the stripe
        pattern of the target.
      rgb2: A sequence of three floating point values between 0.0 and 1.0
        (inclusive) representing the color of the second element in the stripe
        pattern of the target.
      specific_collision_geom_ids: Only activate if collides with these geoms.
      name: The name of this entity.
    """
    self._mjcf_root = mjcf.RootElement(model=name)
    self._texture = self._mjcf_root.asset.add(
        'texture', name='target_sphere', type='cube',
        builtin='checker', rgb1=rgb1, rgb2=rgb2,
        width='100', height='100')
    self._material = self._mjcf_root.asset.add(
        'material', name='target_sphere', texture=self._texture)
    self._geom = self._mjcf_root.worldbody.add(
        'geom', type='sphere', name='geom', gap=2*radius,
        pos=[0, 0, height_above_ground], size=[radius], material=self._material)
    self._geom_id = -1
    self._activated = False
    self._specific_collision_geom_ids = specific_collision_geom_ids

  @property
  def geom(self):
    return self._geom

  @property
  def material(self):
    return self._material

  @property
  def activated(self):
    """Whether this target has been reached during this episode."""
    return self._activated

  def reset(self, physics):
    self._activated = False
    physics.bind(self._material).rgba[-1] = 1

  @property
  def mjcf_model(self):
    return self._mjcf_root

  def initialize_episode_mjcf(self, unused_random_state):
    self._activated = False

  def _update_activation(self, physics):
    if not self._activated:
      for contact in physics.data.contact:
        if self._specific_collision_geom_ids:
          has_specific_collision = (
              contact.geom1 in self._specific_collision_geom_ids or
              contact.geom2 in self._specific_collision_geom_ids)
        else:
          has_specific_collision = True
        if (has_specific_collision and
            self._geom_id in (contact.geom1, contact.geom2)):
          self._activated = True
          physics.bind(self._material).rgba[-1] = 0

  def initialize_episode(self, physics, unused_random_state):
    self._geom_id = physics.model.name2id(self._geom.full_identifier, 'geom')
    self._update_activation(physics)

  def after_substep(self, physics, unused_random_state):
    self._update_activation(physics)


class TargetSphereTwoTouch(composer.Entity):
  """A non-colliding sphere that is activated through touch.

  The target indicates if it has been touched at least once and touched at least
  twice this episode with a two-bit activated state tuple.  It remains activated
  for the remainder of the current episode.

  The target is automatically reset at episode initialization.
  """

  def _build(self,
             radius=0.6,
             height_above_ground=1,
             rgb_initial=((0, 0.4, 0), (0, 0.7, 0)),
             rgb_interval=((1., 1., .4), (0.7, 0.7, 0.)),
             rgb_final=((.4, 0.7, 1.), (0, 0.4, .7)),
             touch_debounce=.2,
             specific_collision_geom_ids=None,
             name='target'):
    """Builds this target sphere.

    Args:
      radius: The radius (in meters) of this target sphere.
      height_above_ground: The height (in meters) of this target above ground.
      rgb_initial: A tuple of two colors for the stripe pattern of the target.
      rgb_interval: A tuple of two colors for the stripe pattern of the target.
      rgb_final: A tuple of two colors for the stripe pattern of the target.
      touch_debounce: duration to not count second touch.
      specific_collision_geom_ids: Only activate if collides with these geoms.
      name: The name of this entity.
    """
    self._mjcf_root = mjcf.RootElement(model=name)
    self._texture_initial = self._mjcf_root.asset.add(
        'texture', name='target_sphere_init', type='cube',
        builtin='checker', rgb1=rgb_initial[0], rgb2=rgb_initial[1],
        width='100', height='100')
    self._texture_interval = self._mjcf_root.asset.add(
        'texture', name='target_sphere_inter', type='cube',
        builtin='checker', rgb1=rgb_interval[0], rgb2=rgb_interval[1],
        width='100', height='100')
    self._texture_final = self._mjcf_root.asset.add(
        'texture', name='target_sphere_final', type='cube',
        builtin='checker', rgb1=rgb_final[0], rgb2=rgb_final[1],
        width='100', height='100')
    self._material = self._mjcf_root.asset.add(
        'material', name='target_sphere_init', texture=self._texture_initial)
    self._geom = self._mjcf_root.worldbody.add(
        'geom', type='sphere', name='geom', gap=2*radius,
        pos=[0, 0, height_above_ground], size=[radius],
        material=self._material)
    self._geom_id = -1
    self._touched_once = False
    self._touched_twice = False
    self._touch_debounce = touch_debounce
    self._specific_collision_geom_ids = specific_collision_geom_ids

  @property
  def geom(self):
    return self._geom

  @property
  def material(self):
    return self._material

  @property
  def activated(self):
    """Whether this target has been reached during this episode."""
    return (self._touched_once, self._touched_twice)

  def reset(self, physics):
    self._touched_once = False
    self._touched_twice = False
    self._geom.material = self._material
    physics.bind(self._material).texid = physics.bind(
        self._texture_initial).element_id

  @property
  def mjcf_model(self):
    return self._mjcf_root

  def initialize_episode_mjcf(self, unused_random_state):
    self._touched_once = False
    self._touched_twice = False

  def _update_activation(self, physics):
    if not (self._touched_once and self._touched_twice):
      for contact in physics.data.contact:
        if self._specific_collision_geom_ids:
          has_specific_collision = (
              contact.geom1 in self._specific_collision_geom_ids or
              contact.geom2 in self._specific_collision_geom_ids)
        else:
          has_specific_collision = True
        if (has_specific_collision and
            self._geom_id in (contact.geom1, contact.geom2)):
          if not self._touched_once:
            self._touched_once = True
            self._touch_time = physics.time()
            physics.bind(self._material).texid = physics.bind(
                self._texture_interval).element_id
          if self._touched_once and (
              physics.time() > (self._touch_time + self._touch_debounce)):
            self._touched_twice = True
            physics.bind(self._material).texid = physics.bind(
                self._texture_final).element_id

  def initialize_episode(self, physics, unused_random_state):
    self._geom_id = physics.model.name2id(self._geom.full_identifier, 'geom')
    self._update_activation(physics)

  def after_substep(self, physics, unused_random_state):
    self._update_activation(physics)



# Copyright 2018 The dm_control Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or  implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

"""Variations in 3D rotations."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from dm_control.composer.variation import base
from dm_control.composer.variation import variation_values
import numpy as np

IDENTITY_QUATERNION = np.array([1., 0., 0., 0.])


class UniformQuaternion(base.Variation):
  """Uniformly distributed unit quaternions."""

  def __call__(self, initial_value=None, current_value=None, random_state=None):
    random_state = random_state or np.random
    u1, u2, u3 = random_state.uniform([0.] * 3, [1., 2. * np.pi, 2. * np.pi])
    return np.array([np.sqrt(1. - u1) * np.sin(u2),
                     np.sqrt(1. - u1) * np.cos(u2),
                     np.sqrt(u1) * np.sin(u3),
                     np.sqrt(u1) * np.cos(u3)])


class QuaternionFromAxisAngle(base.Variation):
  """Quaternion variation specified in terms of variations in axis and angle."""

  def __init__(self, axis, angle):
    self._axis = axis
    self._angle = angle

  def __call__(self, initial_value=None, current_value=None, random_state=None):
    random_state = random_state or np.random
    axis = variation_values.evaluate(
        self._axis, initial_value, current_value, random_state)
    angle = variation_values.evaluate(
        self._angle, initial_value, current_value, random_state)
    sine, cosine = np.sin(angle / 2), np.cos(angle / 2)
    return np.array([cosine, axis[0] * sine, axis[1] * sine, axis[2] * sine])


class QuaternionPreMultiply(base.Variation):
  """A variation that pre-multiplies an existing quaternion value.

  This variation takes a quaternion value generated by another variation and
  pre-multiplies it to an existing value. In cumulative mode, the new quaternion
  is pre-multiplied to the current value being varied. In non-cumulative mode,
  the new quaternion is pre-multiplied to a fixed initial value.
  """

  def __init__(self, quat, cumulative=False):
    self._quat = quat
    self._cumulative = cumulative

  def __call__(self, initial_value=None, current_value=None, random_state=None):
    random_state = random_state or np.random
    q1 = variation_values.evaluate(self._quat, initial_value, current_value,
                                   random_state)
    q2 = current_value if self._cumulative else initial_value
    return np.array([
        q1[0]*q2[0] - q1[1]*q2[1] - q1[2]*q2[2] - q1[3]*q2[3],
        q1[0]*q2[1] + q1[1]*q2[0] + q1[2]*q2[3] - q1[3]*q2[2],
        q1[0]*q2[2] - q1[1]*q2[3] + q1[2]*q2[0] + q1[3]*q2[1],
        q1[0]*q2[3] + q1[1]*q2[2] - q1[2]*q2[1] + q1[3]*q2[0]])
